@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1995, 1996, 1997, 1998, 2001, 2003, 2005, 2007
+Copyright (C) 1995, 1996, 1997, 1998, 2001, 2003, 2005, 2007, 2008, 2009
               John W. Eaton
 
 This file is part of Octave.
@@ -30,9 +30,13 @@ along with Octave; see the file COPYING.  If not, see
 #include "Array.h"
 #include "Array.cc"
 
-INSTANTIATE_ARRAY_AND_ASSIGN (short, OCTAVE_API);
+#define INLINE_ASCENDING_SORT
+#define INLINE_DESCENDING_SORT
+#include "oct-sort.cc"
 
-INSTANTIATE_ARRAY_ASSIGN (short, char, OCTAVE_API);
+INSTANTIATE_ARRAY_SORT (short);
+
+INSTANTIATE_ARRAY (short, OCTAVE_API);
 
 #include "Array2.h"
 
