@@ -2903,7 +2903,7 @@ operator * (const ComplexColumnVector& v, const ComplexRowVector& a)
     {
       octave_idx_type a_len = a.length ();
 
-      retval.resize (len, a_len);
+      retval = ComplexMatrix (len, a_len);
       Complex *c = retval.fortran_vec ();
 
       F77_XFCN (zgemm, ZGEMM, (F77_CONST_CHAR_ARG2 ("N", 1),
@@ -3788,12 +3788,12 @@ xgemm (bool transa, bool conja, const ComplexMatrix& a,
   else
     {
       if (a_nr == 0 || a_nc == 0 || b_nc == 0)
-	retval.resize (a_nr, b_nc, 0.0);
+	retval = ComplexMatrix (a_nr, b_nc, 0.0);
       else if (a.data () == b.data () && a_nr == b_nc && transa != transb)
         {
 	  octave_idx_type lda = a.rows ();
 
-          retval.resize (a_nr, b_nc);
+          retval = ComplexMatrix (a_nr, b_nc);
 	  Complex *c = retval.fortran_vec ();
 
           const char *ctransa = get_blas_trans_arg (transa, conja);
@@ -3829,7 +3829,7 @@ xgemm (bool transa, bool conja, const ComplexMatrix& a,
 	  octave_idx_type lda = a.rows (), tda = a.cols ();
 	  octave_idx_type ldb = b.rows (), tdb = b.cols ();
 
-	  retval.resize (a_nr, b_nc);
+	  retval = ComplexMatrix (a_nr, b_nc);
 	  Complex *c = retval.fortran_vec ();
 
 	  if (b_nc == 1 && a_nr == 1)
