@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-              2007 John W. Eaton
+              2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -50,25 +50,48 @@ along with Octave; see the file COPYING.  If not, see
 #include "quit.h"
 #include "statdefs.h"
 #include "str-vec.h"
+#include "oct-locbuf.h"
+
+file_ops::static_members *file_ops::static_members::instance = 0;
+
+file_ops::static_members::static_members (void)
+  :
+#if (defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM) && ! defined (OCTAVE_HAVE_POSIX_FILESYSTEM))
+  xdir_sep_char ('\\'),
+  xdir_sep_str ("\\"),
+#else
+  xdir_sep_char ('/'),
+  xdir_sep_str ("/"), 
+#endif
+#if defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM)
+  xdir_sep_chars ("/\\")
+#else
+  xdir_sep_chars (xdir_sep_str)
+#endif
+{ }
+
+bool
+file_ops::static_members::instance_ok (void)
+{
+  bool retval = true;
+
+  if (! instance)
+    instance = new static_members ();
+
+  if (! instance)
+    {
+      (*current_liboctave_error_handler)
+	("unable to create file_ops::static_members object!");
+
+      retval = false;
+    }
+
+  return retval;
+}
 
 #define NOT_SUPPORTED(nm) \
   nm ": not supported on this system"
 
-#if (defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM) \
-     && ! defined (OCTAVE_HAVE_POSIX_FILESYSTEM))
-char file_ops::dir_sep_char = '\\';
-std::string file_ops::dir_sep_str ("\\");
-#else
-char file_ops::dir_sep_char = '/';
-std::string file_ops::dir_sep_str ("/");
-#endif
-
-#if defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM)
-std::string file_ops::dir_sep_chars ("/\\");
-#else
-std::string file_ops::dir_sep_chars (file_ops::dir_sep_str);
-#endif
-
 // We provide a replacement for mkdir().
 
 int
@@ -346,7 +369,7 @@ file_ops::recursive_rmdir (const std::string& name, std::string& msg)
 	  if (nm == "." || nm == "..")
 	    continue;
 
-	  std::string fullnm = name + file_ops::dir_sep_str + nm;
+	  std::string fullnm = name + file_ops::dir_sep_str () + nm;
 
 	  // Get info about the file.  Don't follow links.
 	  file_stat fs (fullnm, false);
@@ -467,7 +490,7 @@ extern int errno;
 
   while (true)
     {
-      int status = GetFullPathName (name.c_str (), n, &win_path[0], NULL);
+      int status = GetFullPathName (name.c_str (), n, &win_path[0], 0);
 
       if (status == 0)
         break;
@@ -746,53 +769,58 @@ tilde_expand_word (const std::string& filename)
 std::string
 file_ops::tilde_expand (const std::string& name)
 {
-  std::string result;
-
-  size_t name_len = name.length ();
-
-  // Scan through S expanding tildes as we come to them.
-
-  size_t pos = 0;
-
-  while (1)
+  if (name.find ('~') == std::string::npos)
+    return name;
+  else
     {
-      if (pos > name_len)
-	break;
+      std::string result;
 
-      size_t len;
+      size_t name_len = name.length ();
 
-      // Make START point to the tilde which starts the expansion.
+      // Scan through S expanding tildes as we come to them.
 
-      size_t start = tilde_find_prefix (name.substr (pos), len);
+      size_t pos = 0;
 
-      result.append (name.substr (pos, start));
+      while (1)
+	{
+	  if (pos > name_len)
+	    break;
 
-      // Advance STRING to the starting tilde.
+	  size_t len;
 
-      pos += start;
+	  // Make START point to the tilde which starts the expansion.
 
-      // Make FINI be the index of one after the last character of the
-      // username.
+	  size_t start = tilde_find_prefix (name.substr (pos), len);
 
-      size_t fini = tilde_find_suffix (name.substr (pos));
+	  result.append (name.substr (pos, start));
 
-      // If both START and FINI are zero, we are all done.
+	  // Advance STRING to the starting tilde.
 
-      if (! (start || fini))
-	break;
+	  pos += start;
 
-      // Expand the entire tilde word, and copy it into RESULT.
+	  // Make FINI be the index of one after the last character of the
+	  // username.
 
-      std::string tilde_word = name.substr (pos, fini);
+	  size_t fini = tilde_find_suffix (name.substr (pos));
 
-      pos += fini;
+	  // If both START and FINI are zero, we are all done.
 
-      std::string expansion = tilde_expand_word (tilde_word);
+	  if (! (start || fini))
+	    break;
 
-      result.append (expansion);
+	  // Expand the entire tilde word, and copy it into RESULT.
+
+	  std::string tilde_word = name.substr (pos, fini);
+
+	  pos += fini;
+
+	  std::string expansion = tilde_expand_word (tilde_word);
+
+	  result.append (expansion);
+	}
+
+      return result;
     }
-
-  return result;
 }
 
 // A vector version of the above.
@@ -851,12 +879,6 @@ file_ops::unlink (const std::string& name, std::string& msg)
   return status;
 }
 
-bool
-file_ops::is_dir_sep (char c)
-{
-  return dir_sep_chars.find (c) != NPOS;
-}
-
 std::string
 file_ops::concat (const std::string& dir, const std::string& file)
 {
@@ -864,7 +886,7 @@ file_ops::concat (const std::string& dir, const std::string& file)
     ? file
     : (is_dir_sep (dir[dir.length()-1])
        ? dir + file
-       : dir + file_ops::dir_sep_char + file);
+       : dir + file_ops::dir_sep_char () + file);
 }
 
 /*
