@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2007 John W. Eaton
+Copyright (C) 2004, 2005, 2007, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -24,7 +24,7 @@ along with Octave; see the file COPYING.  If not, see
 #define octave_int16NDArray_h 1
 
 #include "intNDArray.h"
-#include "mx-op-defs.h"
+#include "mx-op-decl.h"
 #include "oct-inttypes.h"
 
 typedef intNDArray<octave_int16> int16NDArray;
