@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+              2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -26,7 +26,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <cstdio>
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 #include "oct-cmplx.h"
@@ -34,8 +34,11 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "syswait.h"
 
 extern OCTAVE_API octave_idx_type NINTbig (double x);
+extern OCTAVE_API octave_idx_type NINTbig (float x);
 extern OCTAVE_API int NINT (double x);
+extern OCTAVE_API int NINT (float x);
 extern OCTAVE_API double D_NINT (double x);
+extern OCTAVE_API float F_NINT (float x);
 
 extern OCTAVE_API char *strsave (const char *);
 
@@ -65,6 +68,12 @@ extern OCTAVE_API Complex octave_read_complex (std::istream& is);
 extern OCTAVE_API void octave_write_double (std::ostream& os, double dval);
 extern OCTAVE_API void octave_write_complex (std::ostream& os, const Complex& cval);
 
+extern OCTAVE_API float octave_read_float (std::istream& is);
+extern OCTAVE_API FloatComplex octave_read_float_complex (std::istream& is);
+
+extern OCTAVE_API void octave_write_float (std::ostream& os, float dval);
+extern OCTAVE_API void octave_write_float_complex (std::ostream& os, const FloatComplex& cval);
+
 #ifdef HAVE_LOADLIBRARY_API
 #include <windows.h>
 extern "C" OCTAVE_API void * octave_w32_library_search (HINSTANCE handle, const char *name);
