@@ -1,4 +1,4 @@
-## Copyright (C) 1997, 2005, 2007 Kurt Hornik
+## Copyright (C) 1997, 2005, 2007, 2008 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -29,8 +29,17 @@
 ## Author: KH <Kurt.Hornik@wu-wien.ac.at>
 ## Description: Random deviates from the hypergeometric distribution
 
+## Deprecated in version 3.0
+
 function inv = hypergeometric_inv (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "hypergeometric_inv is obsolete and will be removed from a future version of Octave; please use hygeinv instead");
+  endif
+
  inv =  hygeinv (varargin{:});
 
 endfunction
