@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2005, 2007 Friedrich Leisch
+## Copyright (C) 1995, 1996, 1997, 2005, 2007, 2008, 2009 Friedrich Leisch
 ##
 ## This file is part of Octave.
 ##
@@ -20,7 +20,7 @@
 ## @deftypefn {Function File} {} wiener_rnd (@var{t}, @var{d}, @var{n})
 ## Return a simulated realization of the @var{d}-dimensional Wiener Process
 ## on the interval [0, @var{t}].  If @var{d} is omitted, @var{d} = 1 is
-## used. The first column of the return matrix contains time, the
+## used.  The first column of the return matrix contains time, the
 ## remaining columns contain the Wiener process.
 ##
 ## The optional parameter @var{n} gives the number of summands used for
@@ -31,8 +31,17 @@
 ## Author: FL <Friedrich.Leisch@ci.tuwien.ac.at>
 ## Description: Simulate a Wiener process
 
+## Deprecated in version 3.0
+
 function retval = wiener_rnd (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "wiener_rnd is obsolete and will be removed from a future version of Octave; please use wienrnd instead");
+  endif
+
  retval =  wienrnd (varargin{:});
 
 endfunction
