@@ -1,4 +1,4 @@
-## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007
+## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007, 2008, 2009
 ##               John W. Eaton
 ##
 ## This file is part of Octave.
@@ -20,6 +20,7 @@
 ## -*- texinfo -*-
 ## @deftypefn {Mapping Function} {} acsch (@var{x})
 ## Compute the inverse hyperbolic cosecant of each element of @var{x}.
+## @seealso{csch}
 ## @end deftypefn
 
 ## Author: jwe
@@ -33,3 +34,13 @@ function w = acsch (z)
   w = asinh (1 ./ z);
 
 endfunction
+
+%!test
+%! v = [pi/2*i, -pi/2*i];
+%! x = [-i, i];
+%! assert(all (abs (acsch (x) - v) < sqrt (eps)));
+
+%!error acsch ();
+
+%!error acsch (1, 2);
+
