@@ -1,4 +1,4 @@
-## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007
+## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007, 2008, 2009
 ##               John W. Eaton
 ##
 ## This file is part of Octave.
@@ -19,7 +19,8 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Mapping Function} {} asec (@var{x})
-## Compute the inverse secant of each element of @var{x}.
+## Compute the inverse secant in radians for each element of @var{x}.
+## @seealso{sec, asecd}
 ## @end deftypefn
 
 ## Author: jwe
@@ -33,3 +34,14 @@ function w = asec (z)
   w = acos (1 ./ z);
 
 endfunction
+
+%!test
+%! rt2 = sqrt (2);
+%! rt3 = sqrt (3);
+%! v = [0, pi/6, pi/4, pi/3, 2*pi/3, 3*pi/4, 5*pi/6, pi];
+%! x = [1, 2*rt3/3, rt2, 2, -2, -rt2, -2*rt3/3, -1];
+%! assert(all (abs (asec (x) - v) < sqrt (eps)));
+
+%!error asec ();
+
+%!error asec (1, 2);
