@@ -1,4 +1,4 @@
-## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007
+## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007, 2008, 2009
 ##               John W. Eaton
 ##
 ## This file is part of Octave.
@@ -19,7 +19,8 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Mapping Function} {} cot (@var{x})
-## Compute the cotangent of each element of @var{x}.
+## Compute the cotangent for each element of @var{x} in radians.
+## @seealso{acot, cotd, coth}
 ## @end deftypefn
 
 ## Author: jwe
@@ -33,3 +34,15 @@ function w = cot (z)
   w = 1 ./ tan(z);
 
 endfunction
+
+%!test
+%! rt2 = sqrt (2);
+%! rt3 = sqrt (3);
+%! x = [pi/6, pi/4, pi/3, pi/2, 2*pi/3, 3*pi/4, 5*pi/6];
+%! v = [rt3, 1, rt3/3, 0, -rt3/3, -1, -rt3];
+%! assert(all (abs (cot (x) - v) < sqrt (eps)));
+
+%!error cot ();
+
+%!error cot (1, 2);
+
