@@ -1,4 +1,4 @@
-## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007
+## Copyright (C) 1994, 1996, 1997, 1999, 2000, 2005, 2006, 2007, 2008, 2009
 ##               John W. Eaton
 ##
 ## This file is part of Octave.
@@ -20,6 +20,7 @@
 ## -*- texinfo -*-
 ## @deftypefn {Mapping Function} {} csch (@var{x})
 ## Compute the hyperbolic cosecant of each element of @var{x}.
+## @seealso{acsch}
 ## @end deftypefn
 
 ## Author: jwe
@@ -33,3 +34,13 @@ function w = csch (z)
   w = 1 ./ sinh(z);
 
 endfunction
+
+%!test
+%! x = [pi/2*i, 3*pi/2*i];
+%! v = [-i, i];
+%! assert(all (abs (csch (x) - v) < sqrt (eps)));
+
+%!error csch ();
+
+%!error csch (1, 2);
+
