@@ -1,5 +1,5 @@
 ## Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2003, 2005,
-##               2006, 2007 John W. Eaton
+##               2006, 2007, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -38,8 +38,6 @@
 ## Created: July 1994
 ## Adapted-By: jwe
 
-## PKG_ADD: mark_as_command colormap
-
 function cmap = colormap (map)
 
   if (nargin > 1)
