@@ -1,5 +1,5 @@
 ## Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2004, 2005,
-##               2006, 2007, 2008 John W. Eaton
+##               2006, 2007, 2008, 2009 John W. Eaton
 ## 
 ## This file is part of Octave.
 ##
@@ -40,7 +40,7 @@
 ## shown.
 ##
 ## If given, the parameter @var{string_param1} has value
-## @var{value1}. @var{string_param1} can be any of the following:
+## @var{value1}.  @var{string_param1} can be any of the following:
 ## @table @samp
 ## @item "displayrange"
 ## @var{value1} is the display range as described above.
@@ -65,7 +65,7 @@ function h = imshow (im, varargin)
   ## Get the image.
   if (ischar (im))
     ## Eventually, this should be imread.
-    [im, map] = loadimage (im);
+    [im, map] = imread (im);
     indexed = true;
     colormap (map);
   endif
@@ -103,7 +103,7 @@ function h = imshow (im, varargin)
     elseif (ischar (arg))
       switch (arg)
 	case "displayrange";
-	  displayrange = varargin{narg++};
+	  display_range = varargin{narg++};
 	case {"truesize", "initialmagnification"}
 	  warning ("image: zoom argument ignored -- use GUI features");
 	otherwise
@@ -115,10 +115,10 @@ function h = imshow (im, varargin)
     endif
   endwhile
 
-  ## Set default display range.
+  ## Set default display range if display_range not set yet.
   if (isempty (display_range))
     display_range = [min(im(:)), max(im(:))];
-  else
+  elseif (isna (display_range))
     t = class (im);
     switch (t)
       case {"double", "single", "logical"}
@@ -147,7 +147,7 @@ function h = imshow (im, varargin)
   endif
 
   ## This is for compatibility.
-  if (! indexed || islogical (im))
+  if (! (indexed || (true_color && isinteger (im))) || islogical (im))
     im = double (im);
   endif
 
@@ -160,8 +160,8 @@ function h = imshow (im, varargin)
     im(im > 1) = 1;
   endif
 
-  if (true_color)
-    tmp = __img__ ([], [], im);
+  if (true_color || indexed)
+    tmp = image ([], [], im);
   else
     tmp = image (round ((rows (colormap ()) - 1) * im));
   endif
@@ -186,12 +186,13 @@ endfunction
 %!  colormap ("autumn");
 
 %!demo
-%!  [I, M] = loadimage ("default.img");
+%!  [I, M] = imread ("default.img");
 %!  imshow (I, M);
 
 %!demo
-%!  [I, M] = loadimage ("default.img");
-%!  imshow (cat(3, I, I*0.5, I*0.8));
+%!  [I, M] = imread ("default.img");
+%!  [R, G, B] = ind2rgb (I, M);
+%!  imshow (cat(3, R, G*0.5, B*0.8));
 
 %!demo
 %!  imshow (rand (100, 100));
