@@ -1,4 +1,5 @@
-## Copyright (C) 1996, 1997, 1998, 2000, 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 1996, 1997, 1998, 2000, 2005, 2006, 2007, 2009
+##               John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,7 +19,7 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} flops ()
-## This function is provided for @sc{Matlab} compatibility, but it doesn't
+## This function is provided for @sc{matlab} compatibility, but it doesn't
 ## actually do anything.
 ## @end deftypefn
 
