@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 Sren Hauberg
+## Copyright (C) 2005, 2006, 2007, 2008 Sren Hauberg
 ## 
 ## This file is part of Octave.
 ##
@@ -20,7 +20,7 @@
 ## @deftypefn {Function File} {} unzip (@var{zipfile}, @var{dir})
 ## Unpack the ZIP archive @var{zipfile} to the directory @var{dir}.
 ## If @var{dir} is not specified, it defaults to the current directory.
-## @seealso{unpack, bzip2, bunzip2, tar, untar, gzip, gunzip, zip}
+## @seealso{unpack, bunzip2, tar, untar, gzip, gunzip, zip}
 ## @end deftypefn
 
 ## Author: Sren Hauberg <hauberg@gmail.com>
