@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 2005, 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -28,8 +28,12 @@
 function closereq ()
 
   if (nargin == 0)
-    cf = get (0, "currentfigure");
-    if (cf != 0)
+    cf = gcbf ();
+    if (isempty (cf))
+      warning ("closereq: calling closereq from octave prompt is not supported, use `close' instead");
+      cf = get (0, "currentfigure");
+    endif
+    if (! isempty (cf) && isfigure (cf))
       delete (cf);
     endif
   else
