@@ -1,4 +1,4 @@
-## Copyright (C) 2001, 2006, 2007 Laurent Mazet
+## Copyright (C) 2001, 2006, 2007, 2008, 2009 Laurent Mazet
 ## Copyright (C) 2006 John W. Eaton
 ##
 ## This file is part of Octave.
@@ -76,14 +76,11 @@
 ## @end table
 ## @end deftypefn
 
-## PKG_ADD mark_as_command legend
-
 function legend (varargin)
 
+  [ca, varargin, nargin] = __plt_get_axis_arg__ ("legend", varargin{:});
   nargs = nargin;
 
-  ca = gca ();
-
   if (nargs > 0)
     pos = varargin{nargs};
     if (isnumeric (pos) && isscalar (pos) && round (pos) == pos)
@@ -122,7 +119,7 @@ function legend (varargin)
 	    set (ca, "key", "on");
 	  case "toggle"
 	    val = get (ca, "key");
-	    if (strcmp (val, "on"))
+	    if (strcmpi (val, "on"))
 	      set (ca, "key", "off");
 	    else
 	      set (ca, "key", "on");
@@ -131,13 +128,31 @@ function legend (varargin)
 	    set (ca, "key", "on", "keybox", "on");
 	  case "boxoff"
 	    set (ca, "keybox", "off");
+	  case "left"
+	    set (ca, "keyreverse", "off")
+	  case "right"
+	    set (ca, "keyreverse", "on")
 	  otherwise
-	    while (k <= nkids && ! strcmp (get (kids(k), "type"), "line"))
+	    typ = get (kids (k), "type");
+	    while (k <= nkids && ! strcmp (typ, "line") &&
+		   ! strcmp (typ, "hggroup"))
 	      k++;
+	      typ = get (kids (k), "type");
 	    endwhile
 	    if (k <= nkids)
 	      turn_on_legend = true;
-	      set (kids(k), "keylabel", arg);
+	      if (strcmp (typ, "hggroup"))
+		hgkids = get (kids(k), "children");
+		for j = 1 : length (hgkids)
+		  hgobj = get (hgkids (j));
+		  if (isfield (hgobj, "keylabel"))
+		    set (hgkids(j), "keylabel", arg);
+		    break;
+		  endif
+		endfor
+	      else
+		set (kids(k), "keylabel", arg);
+	      endif
 	    else
 	      warning ("legend: ignoring extra labels");
 	    endif
@@ -160,7 +175,8 @@ function legend (varargin)
     for i = 1:nkids
       if (strcmp (get (kids(k), "type"), "line")
 	  || strcmp (get (kids(k), "type"), "surface")
-	  || strcmp (get (kids(k), "type"), "patch"))
+	  || strcmp (get (kids(k), "type"), "patch")
+	  || strcmp (get (kids(k), "type"), "hggroup"))
 	have_data = true;
 	break;
       endif
@@ -171,17 +187,29 @@ function legend (varargin)
   endif
 
   warned = false;
-  for i = 1:nargs
+  for i = nargs:-1:1
     arg = varargin{i};
     if (ischar (arg))
       while (k <= nkids
 	     && ! (strcmp (get (kids(k), "type"), "line")
 		   || strcmp (get (kids(k), "type"), "surface")
-		   || strcmp (get (kids(k), "type"), "patch")))
+		   || strcmp (get (kids(k), "type"), "patch")
+		   || strcmp (get (kids(k), "type"), "hggroup")))
 	k++;
       endwhile
       if (k <= nkids)
-	set (kids(k), "keylabel", arg);
+	if (strcmp (get (kids(k), "type"), "hggroup"))
+	  hgkids = get (kids(k), "children");
+	  for j = 1 : length (hgkids)
+	    hgobj = get (hgkids (j));
+	    if (isfield (hgobj, "keylabel"))
+	      set (hgkids(j), "keylabel", arg);
+	      break;
+	    endif
+	  endfor
+	else
+	  set (kids(k), "keylabel", arg);
+	endif
 	turn_on_legend = true;
 	k++;
       elseif (! warned)
@@ -200,24 +228,24 @@ function legend (varargin)
 endfunction
 
 %!demo
-%! close all;
-%! plot(1:10, 1:10, 'b', 1:10, fliplr(1:10), 'g');
+%! clf
+%! plot(1:10, 1:10, 1:10, fliplr(1:10));
 %! title("incline is blue and decline is green");
 %! legend({"I'm blue", "I'm green"}, "location", "east")
 
 %!demo
-%! close all;
+%! clf
 %! plot(1:10, 1:10);
 %! title("a very long label can sometimes cause problems");
 %! legend({"hello world"}, "location", "northeastoutside")
 
 %!demo
-%! close all;
+%! clf
 %! labels = {};
-%! color = 'bgryk';
 %! for i = 1:5
-%!     plot(1:100, i + rand(100,1), color(i)); hold on;
-%!     labels = {labels{:}, strcat("Signal ", num2str(i))};
+%!     h = plot(1:100, i + rand(100,1)); hold on;
+%!     set (h, "color", get (gca, "colororder")(i,:))
+%!     labels = {labels{:}, cstrcat("Signal ", num2str(i))};
 %! endfor; hold off;
 %! title("Signals with random offset and uniform noise")
 %! xlabel("Sample Nr [k]"); ylabel("Amplitude [V]");
