@@ -1,5 +1,5 @@
 ## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2003,
-##               2005, 2006, 2007, 2008 John W. Eaton
+##               2005, 2006, 2007, 2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -20,11 +20,9 @@
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} ylabel (@var{string})
 ## @deftypefnx {Function File} {} ylabel (@var{h}, @var{string})
-## See xlabel.
+## @seealso{xlabel}.
 ## @end deftypefn
 
-## PKG_ADD: mark_as_command ylabel
-
 ## Author: jwe
 
 function retval = ylabel (varargin)
