@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007
+## Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007, 2009
 ##               Andreas Weingessel
 ##
 ## This file is part of Octave.
@@ -21,8 +21,8 @@
 ## @deftypefn {Function File} {} hanning (@var{m})
 ## Return the filter coefficients of a Hanning window of length @var{m}.
 ##
-## For a definition of this window type, see e.g. A. V. Oppenheim &
-## R. W. Schafer, "Discrete-Time Signal Processing".
+## For a definition of this window type, see e.g., A. V. Oppenheim &
+## R. W. Schafer, @cite{Discrete-Time Signal Processing}.
 ## @end deftypefn
 
 ## Author: AW <Andreas.Weingessel@ci.tuwien.ac.at>
