@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2006, 2007 Kurt Hornik
+## Copyright (C) 1995, 1996, 1997, 2006, 2007, 2009 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -22,11 +22,9 @@
 ## Weibull distribution with shape parameter @var{scale} and scale
 ## parameter @var{shape} which is given by
 ##
-## @iftex
 ## @tex
 ## $$  scale \cdot shape^{-scale} x^{scale-1} \exp(-(x/shape)^{scale}) $$
 ## @end tex
-## @end iftex
 ## @ifnottex
 ## @example
 ##    scale * shape^(-scale) * x^(scale-1) * exp(-(x/shape)^scale)
