@@ -1,4 +1,4 @@
-## Copyright (C) 1996, 1997, 1999, 2002, 2003, 2005, 2006, 2007
+## Copyright (C) 1996, 1997, 1999, 2002, 2003, 2005, 2006, 2007, 2008, 2009
 ##               Kurt Hornik
 ##
 ## This file is part of Octave.
@@ -19,7 +19,18 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} blanks (@var{n})
-## Return a string of @var{n} blanks.
+## Return a string of @var{n} blanks, for example:
+##
+## @example
+## @group
+## blanks(10);
+## whos ans;
+##      @result{}
+##       Attr Name        Size                     Bytes  Class
+##       ==== ====        ====                     =====  ===== 
+##            ans         1x10                        10  char
+## @end group
+## @end example
 ## @seealso{repmat}
 ## @end deftypefn
 
@@ -44,3 +55,10 @@ endfunction
 %!assert(blanks (0), "")
 %!assert(blanks (5), "     ")
 %!assert(blanks (10), "          ")
+
+%!assert(strcmp (blanks (3), "   "));
+
+%!error blanks ();
+
+%!error blanks (1, 2);
+
