@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-              2005, 2006, 2007 John W. Eaton
+              2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -187,7 +187,7 @@ with\n\
 $$ x(t_0) = x_0, \\dot{x}(t_0) = \\dot{x}_0 $$\n\
 @end tex\n\
 @end iftex\n\
-@ifinfo\n\
+@ifnottex\n\
 \n\
 @example\n\
 0 = f (x, xdot, t)\n\
@@ -200,7 +200,7 @@ with\n\
 x(t_0) = x_0, xdot(t_0) = xdot_0\n\
 @end example\n\
 \n\
-@end ifinfo\n\
+@end ifnottex\n\
 The solution is returned in the matrices @var{x} and @var{xdot},\n\
 with each row in the result matrices corresponding to one of the\n\
 elements in the vector @var{t}.  The first element of @var{t}\n\
@@ -209,10 +209,9 @@ system @var{x_0} and its derivative @var{xdot_0}, so that the first\n\
 row of the output @var{x} is @var{x_0} and the first row\n\
 of the output @var{xdot} is @var{xdot_0}.\n\
 \n\
-The first argument, @var{fcn}, is a string or a two element cell array\n\
-of strings, inline or function handle, that names the function, to call\n\
-to compute the vector of residuals for the set of equations. It must\n\
-have the form\n\
+The first argument, @var{fcn}, is a string, inline, or function handle\n\
+that names the function @math{f} to call to compute the vector of\n\
+residuals for the set of equations.  It must have the form\n\
 \n\
 @example\n\
 @var{res} = f (@var{x}, @var{xdot}, @var{t})\n\
@@ -222,9 +221,10 @@ have the form\n\
 in which @var{x}, @var{xdot}, and @var{res} are vectors, and @var{t} is a\n\
 scalar.\n\
 \n\
-If @var{fcn} is a two-element string array, the first element names\n\
-the function @math{f} described above, and the second element names\n\
-a function to compute the modified Jacobian\n\
+If @var{fcn} is a two-element string array or a two-element cell array\n\
+of strings, inline functions, or function handles, the first element names\n\
+the function @math{f} described above, and the second element names a\n\
+function to compute the modified Jacobian\n\
 \n\
 @iftex\n\
 @tex\n\
@@ -234,20 +234,24 @@ J = {\\partial f \\over \\partial x}\n\
 $$\n\
 @end tex\n\
 @end iftex\n\
-@ifinfo\n\
+@ifnottex\n\
 @example\n\
+@group\n\
       df       df\n\
 jac = -- + c ------\n\
       dx     d xdot\n\
+@end group\n\
 @end example\n\
-@end ifinfo\n\
+@end ifnottex\n\
 \n\
 The modified Jacobian function must have the form\n\
 \n\
 @example\n\
+@group\n\
 \n\
 @var{jac} = j (@var{x}, @var{xdot}, @var{t}, @var{c})\n\
 \n\
+@end group\n\
 @end example\n\
 \n\
 The second and third arguments to @code{dassl} specify the initial\n\
@@ -487,6 +491,87 @@ parameters for @code{dassl}.\n\
   return retval;
 }
 
+/*
+
+%% dassl-1.m
+%%
+%% Test dassl() function
+%%
+%% Author: David Billinghurst (David.Billinghurst@riotinto.com.au)
+%%         Comalco Research and Technology
+%%         20 May 1998
+%%
+%% Problem
+%%
+%%    y1' = -y2,   y1(0) = 1
+%%    y2' =  y1,   y2(0) = 0
+%%
+%% Solution
+%%
+%%    y1(t) = cos(t)
+%%    y2(t) = sin(t)
+%!function res = f (x, xdot, t)
+%!  res = [xdot(1)+x(2); xdot(2)-x(1)];
+%!test
+%! 
+%! x0 = [1; 0];
+%! xdot0 = [0; 1];
+%! t = (0:1:10)';
+%! 
+%! tol = 100 * dassl_options ("relative tolerance");
+%! 
+%! 
+%! [x, xdot] = dassl ("f", x0, xdot0, t);
+%! 
+%! y = [cos(t), sin(t)];
+%! 
+%! assert(all (all (abs (x - y) < tol)));
+
+%% dassl-2.m
+%%
+%% Test dassl() function
+%%
+%% Author: David Billinghurst (David.Billinghurst@riotinto.com.au)
+%%         Comalco Research and Technology
+%%         20 May 1998
+%%
+%% Based on SLATEC quick check for DASSL by Linda Petzold
+%%
+%% Problem
+%%
+%%   x1' + 10*x1 = 0,   x1(0) = 1
+%%   x1  + x2    = 1,   x2(0) = 0
+%% 
+%%
+%% Solution
+%%
+%%  x1(t) = exp(-10*t)
+%%  x2(t) = 1 - x(1)
+%!function res = f (x, xdot, t)
+%!  res = [xdot(1)+10*x(1); x(1)+x(2)-1];
+%!test
+%! 
+%! x0 = [1; 0];
+%! xdot0 = [-10; 10];
+%! t = (0:0.2:1)';
+%! 
+%! tol = 500 * dassl_options ("relative tolerance");
+%! 
+%! 
+%! [x, xdot] = dassl ("f", x0, xdot0, t);
+%! 
+%! y = [exp(-10*t), 1-exp(-10*t)];
+%! 
+%! assert(all (all (abs (x - y) < tol)));
+
+%!test
+%! dassl_options ("absolute tolerance", eps);
+%! assert(dassl_options ("absolute tolerance") == eps);
+
+%!error <Invalid call to dassl_options.*> dassl_options ("foo", 1, 2);
+
+*/
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
