@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 David Bateman
 
 This file is part of Octave.
 
@@ -35,9 +35,9 @@ along with Octave; see the file COPYING.  If not, see
 // This function should be merged with Fifft.
 
 #if defined (HAVE_FFTW3)
-#define FFTSRC "@sc{Fftw}"
+#define FFTSRC "@sc{fftw}"
 #else
-#define FFTSRC "@sc{Fftpack}"
+#define FFTSRC "@sc{fftpack}"
 #endif
 
 static octave_value
@@ -88,31 +88,62 @@ do_fftn (const octave_value_list &args, const char *fcn, int type)
     return retval;
 
   if (dims.all_zero ())
-    return octave_value (Matrix ());
-
-  if (arg.is_real_type ())
     {
-      NDArray nda = arg.array_value ();
-
-      if (! error_state)
-	{
-	  nda.resize (dims, 0.0);
-	  retval = (type != 0 ? nda.ifourierNd () : nda.fourierNd ());
-	}
+      if (arg.is_single_type ())
+	return octave_value (FloatMatrix ());
+      else
+	return octave_value (Matrix ());
     }
-  else if (arg.is_complex_type ())
-    {
-      ComplexNDArray cnda = arg.complex_array_value ();
 
-      if (! error_state)
+  if (arg.is_single_type ())
+    {
+      if (arg.is_real_type ())
 	{
-	  cnda.resize (dims, 0.0);
-	  retval = (type != 0 ? cnda.ifourierNd () : cnda.fourierNd ());
+	  FloatNDArray nda = arg.float_array_value ();
+
+	  if (! error_state)
+	    {
+	      nda.resize (dims, 0.0);
+	      retval = (type != 0 ? nda.ifourierNd () : nda.fourierNd ());
+	    }
+	}
+      else
+	{
+	  FloatComplexNDArray cnda = arg.float_complex_array_value ();
+
+	  if (! error_state)
+	    {
+	      cnda.resize (dims, 0.0);
+	      retval = (type != 0 ? cnda.ifourierNd () : cnda.fourierNd ());
+	    }
 	}
     }
   else
     {
-      gripe_wrong_type_arg (fcn, arg);
+      if (arg.is_real_type ())
+	{
+	  NDArray nda = arg.array_value ();
+
+	  if (! error_state)
+	    {
+	      nda.resize (dims, 0.0);
+	      retval = (type != 0 ? nda.ifourierNd () : nda.fourierNd ());
+	    }
+	}
+      else if (arg.is_complex_type ())
+	{
+	  ComplexNDArray cnda = arg.complex_array_value ();
+
+	  if (! error_state)
+	    {
+	      cnda.resize (dims, 0.0);
+	      retval = (type != 0 ? cnda.ifourierNd () : cnda.fourierNd ());
+	    }
+	}
+      else
+	{
+	  gripe_wrong_type_arg (fcn, arg);
+	}
     }
 
   return retval;
@@ -121,12 +152,12 @@ do_fftn (const octave_value_list &args, const char *fcn, int type)
 DEFUN_DLD (fftn, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {} fftn (@var{a}, @var{size})\n\
-Compute the N dimensional FFT of @var{a} using subroutines from\n"
+Compute the N-dimensional FFT of @var{a} using subroutines from\n"
 FFTSRC
-". The optional vector argument @var{size} may be used specify the\n\
-dimensions of the array to be used. If an element of @var{size} is\n\
+".  The optional vector argument @var{size} may be used specify the\n\
+dimensions of the array to be used.  If an element of @var{size} is\n\
 smaller than the corresponding dimension, then the dimension is\n\
-truncated prior to performing the FFT. Otherwise if an element\n\
+truncated prior to performing the FFT.  Otherwise if an element\n\
 of @var{size} is larger than the corresponding dimension @var{a}\n\
 is resized and padded with zeros.\n\
 @seealso {ifftn, fft, fft2, fftw}\n\
@@ -138,12 +169,12 @@ is resized and padded with zeros.\n\
 DEFUN_DLD (ifftn, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {} ifftn (@var{a}, @var{size})\n\
-Compute the inverse N dimensional FFT of @var{a} using subroutines from\n"
+Compute the inverse N-dimensional FFT of @var{a} using subroutines from\n"
 FFTSRC
-". The optional vector argument @var{size} may be used specify the\n\
-dimensions of the array to be used. If an element of @var{size} is\n\
+".  The optional vector argument @var{size} may be used specify the\n\
+dimensions of the array to be used.  If an element of @var{size} is\n\
 smaller than the corresponding dimension, then the dimension is\n\
-truncated prior to performing the inverse FFT. Otherwise if an element\n\
+truncated prior to performing the inverse FFT.  Otherwise if an element\n\
 of @var{size} is larger than the corresponding dimension @var{a}\n\
 is resized and padded with zeros.\n\
 @seealso {fftn, ifft, ifft2, fftw}\n\
