@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -264,8 +264,8 @@ along with Octave; see the file COPYING.  If not, see
 DEFUN (bitand, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} bitand (@var{x}, @var{y})\n\
-Return the bitwise AND of nonnegative integers.\n\
-@var{x}, @var{y} must be in range [0..bitmax]\n\
+Return the bitwise AND of non-negative integers.\n\
+@var{x}, @var{y} must be in the range [0,bitmax]\n\
 @seealso{bitor, bitxor, bitset, bitget, bitcmp, bitshift, bitmax}\n\
 @end deftypefn")
 {
@@ -275,8 +275,8 @@ Return the bitwise AND of nonnegative integers.\n\
 DEFUN (bitor, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} bitor (@var{x}, @var{y})\n\
-Return the bitwise OR of nonnegative integers.\n\
-@var{x}, @var{y} must be in range [0..bitmax]\n\
+Return the bitwise OR of non-negative integers.\n\
+@var{x}, @var{y} must be in the range [0,bitmax]\n\
 @seealso{bitor, bitxor, bitset, bitget, bitcmp, bitshift, bitmax}\n\
 @end deftypefn")
 {
@@ -286,8 +286,8 @@ Return the bitwise OR of nonnegative integers.\n\
 DEFUN (bitxor, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} bitxor (@var{x}, @var{y})\n\
-Return the bitwise XOR of nonnegative integers.\n\
-@var{x}, @var{y} must be in range [0..bitmax]\n\
+Return the bitwise XOR of non-negative integers.\n\
+@var{x}, @var{y} must be in the range [0,bitmax]\n\
 @seealso{bitand, bitor, bitset, bitget, bitcmp, bitshift, bitmax}\n\
 @end deftypefn")
 {
@@ -309,6 +309,21 @@ bitshift (double a, int n, int64_t mask)
     return static_cast<int64_t> (a) & mask;
 }
 
+static int64_t
+bitshift (float a, int n, int64_t mask)
+{
+  // In the name of bug-for-bug compatibility.
+  if (a < 0)
+    return -bitshift (-a, n, mask);
+
+  if (n > 0)
+    return (static_cast<int64_t> (a) << n) & mask;
+  else if (n < 0)
+    return (static_cast<int64_t> (a) >> -n) & mask;
+  else
+    return static_cast<int64_t> (a) & mask;
+}
+
 // Note that the bitshift operators are undefined if shifted by more
 // bits than in the type, so we need to test for the size of the
 // shift.
@@ -393,13 +408,14 @@ DEFUN (bitshift, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} bitshift (@var{a}, @var{k})\n\
 @deftypefnx {Built-in Function} {} bitshift (@var{a}, @var{k}, @var{n})\n\
-Return a @var{k} bit shift of @var{n}- digit unsigned\n\
+Return a @var{k} bit shift of @var{n}-digit unsigned\n\
 integers in @var{a}.  A positive @var{k} leads to a left shift.\n\
 A negative value to a right shift.  If @var{n} is omitted it defaults\n\
 to log2(bitmax)+1.\n\
-@var{n} must be in range [1,log2(bitmax)+1] usually [1,33]\n\
+@var{n} must be in the range [1,log2(bitmax)+1] usually [1,33]\n\
 \n\
 @example\n\
+@group\n\
 bitshift (eye (3), 1)\n\
 @result{}\n\
 @group\n\
@@ -415,6 +431,7 @@ bitshift (10, [-2, -1, 0, 1, 2])\n\
 @c \n\
 @c bitshift ([1, 10], 2, [3,4])\n\
 @c @result{} 4  8\n\
+@end group\n\
 @end example\n\
 @seealso{bitand, bitor, bitxor, bitset, bitget, bitcmp, bitmax}\n\
 @end deftypefn")
@@ -498,7 +515,7 @@ DEFUN (bitmax, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} bitmax ()\n\
 Return the largest integer that can be represented as a floating point\n\
-value.  On IEEE-754 compatiable systems, @code{bitmax} is @code{2^53 - 1}.\n\
+value.  On IEEE-754 compatible systems, @code{bitmax} is @code{2^53 - 1}.\n\
 @end deftypefn")
 {
   octave_value retval;
