@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2009
               John W. Eaton
 
 This file is part of Octave.
@@ -25,6 +25,8 @@ along with Octave; see the file COPYING.  If not, see
 #include <config.h>
 #endif
 
+#include <iostream>
+
 #include "c-file-ptr-stream.h"
 
 #ifndef SEEK_SET
@@ -126,7 +128,8 @@ seekdir_to_whence (std::ios::seekdir dir)
 }
 
 std::streampos
-c_file_ptr_buf::seekoff (std::streamoff offset, std::ios::seekdir dir,
+c_file_ptr_buf::seekoff (std::streamoff /* offset */,
+			 std::ios::seekdir /* dir */,
 			 std::ios::openmode)
 {
   // FIXME
@@ -144,7 +147,7 @@ c_file_ptr_buf::seekoff (std::streamoff offset, std::ios::seekdir dir,
 }
 
 std::streampos
-c_file_ptr_buf::seekpos (std::streampos offset, std::ios::openmode)
+c_file_ptr_buf::seekpos (std::streampos /* offset */, std::ios::openmode)
 {
   // FIXME
 #if 0  
@@ -270,8 +273,9 @@ c_zfile_ptr_buf::xsgetn (char *s, std::streamsize n)
 }
 
 std::streampos
-c_zfile_ptr_buf::seekoff (std::streamoff offset, std::ios::seekdir dir,
-			 std::ios::openmode)
+c_zfile_ptr_buf::seekoff (std::streamoff /* offset */,
+			  std::ios::seekdir /* dir */,
+			  std::ios::openmode)
 {
   // FIXME
 #if 0
@@ -288,7 +292,7 @@ c_zfile_ptr_buf::seekoff (std::streamoff offset, std::ios::seekdir dir,
 }
 
 std::streampos
-c_zfile_ptr_buf::seekpos (std::streampos offset, std::ios::openmode)
+c_zfile_ptr_buf::seekpos (std::streampos /* offset */, std::ios::openmode)
 {
   // FIXME
 #if 0  
