@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -33,6 +33,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <cassert>
 
 #include <iostream>
+#include <sstream>
 #include <string>
 
 #ifdef HAVE_UNISTD_H
@@ -47,12 +48,14 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "quit.h"
 #include "str-vec.h"
 
+#include "debug.h"
 #include "defun.h"
 #include "dirfns.h"
 #include "error.h"
 #include "gripes.h"
 #include "help.h"
 #include "input.h"
+#include "load-path.h"
 #include "oct-map.h"
 #include "oct-hist.h"
 #include "toplev.h"
@@ -62,22 +65,23 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "pathlen.h"
 #include "pt.h"
 #include "pt-const.h"
+#include "pt-eval.h"
 #include "pt-stmt.h"
 #include "sighandlers.h"
-#include "symtab.h"
 #include "sysdep.h"
+#include "toplev.h"
 #include "unwind-prot.h"
 #include "utils.h"
 #include "variables.h"
 
 // Primary prompt string.
-static std::string Vps1;
+static std::string VPS1 = "\\s:\\#> ";
 
 // Secondary prompt string.
-static std::string Vps2;
+static std::string VPS2 = "> ";
 
 // String printed before echoed input (enabled by --echo-input).
-std::string Vps4;
+std::string VPS4 = "+ ";
 
 // Echo commands as they are executed?
 //
@@ -86,13 +90,13 @@ std::string Vps4;
 //   4  ==>  echo commands read from command line
 //
 // more than one state can be active at once.
-int Vecho_executing_commands;
+int Vecho_executing_commands = ECHO_OFF;
 
 // The time we last printed a prompt.
-octave_time Vlast_prompt_time;
+octave_time Vlast_prompt_time = 0.0;
 
 // Character to append after successful command-line completion attempts.
-static char Vcompletion_append_char;
+static char Vcompletion_append_char = ' ';
 
 // Global pointer for eval().
 std::string current_eval_string;
@@ -138,6 +142,23 @@ std::string current_input_line;
 // TRUE after a call to completion_matches.
 bool octave_completion_matches_called = false;
 
+// TRUE if the plotting system has requested a call to drawnow at
+// the next user prompt.
+bool Vdrawnow_requested = false;
+
+// TRUE if we are in debugging mode.
+bool Vdebugging = false;
+
+// If we are in debugging mode, this is the last command entered, so
+// that we can repeat the previous command if the user just types RET.
+static std::string last_debugging_command;
+
+// TRUE if we are running in the Emacs GUD mode.
+static bool Vgud_mode = false;
+
+// The filemarker used to separate filenames from subfunction names
+char Vfilemarker = '>';
+
 static void
 do_input_echo (const std::string& input_string)
 {
@@ -150,12 +171,12 @@ do_input_echo (const std::string& input_string)
       if (forced_interactive)
 	{
 	  if (promptflag > 0)
-	    octave_stdout << command_editor::decode_prompt_string (Vps1);
+	    octave_stdout << command_editor::decode_prompt_string (VPS1);
 	  else
-	    octave_stdout << command_editor::decode_prompt_string (Vps2);
+	    octave_stdout << command_editor::decode_prompt_string (VPS2);
 	}
       else
-	octave_stdout << command_editor::decode_prompt_string (Vps4);
+	octave_stdout << command_editor::decode_prompt_string (VPS4);
 
       if (! input_string.empty ())
 	{
@@ -189,7 +210,7 @@ gnu_readline (const std::string& s, bool force_readline)
 	{
 	  FILE *stream = command_editor::get_output_stream ();
 
-	  fprintf (stream, s.c_str ());
+	  fputs (s.c_str (), stream);
 	  fflush (stream);
 	}
 
@@ -204,6 +225,29 @@ gnu_readline (const std::string& s, bool force_readline)
   return retval;
 }
 
+static inline std::string
+interactive_input (const std::string& s, bool force_readline = false)
+{
+  Vlast_prompt_time.stamp ();
+
+  if (Vdrawnow_requested && (interactive || forced_interactive))
+    {
+      feval ("drawnow");
+
+      flush_octave_stdout ();
+
+      // We set Vdrawnow_requested to false even if there is an error
+      // in drawnow so that the error doesn't reappear at every prompt.
+
+      Vdrawnow_requested = false;
+
+      if (error_state)
+	return "\n";
+    }
+
+  return gnu_readline (s, force_readline);
+}
+
 static std::string
 octave_gets (void)
 {
@@ -211,12 +255,15 @@ octave_gets (void)
 
   std::string retval;
 
-  Vlast_prompt_time.stamp ();
+  bool history_skip_auto_repeated_debugging_command = false;
 
   if ((interactive || forced_interactive)
-      && (! (reading_fcn_file || reading_script_file)))
+      && (! (reading_fcn_file
+	     || reading_script_file
+	     || input_from_startup_file
+	     || input_from_command_line_file)))
     {
-      std::string ps = (promptflag > 0) ? Vps1 : Vps2;
+      std::string ps = (promptflag > 0) ? VPS1 : VPS2;
 
       std::string prompt = command_editor::decode_prompt_string (ps);
 
@@ -226,7 +273,25 @@ octave_gets (void)
 
       octave_diary << prompt;
 
-      retval = gnu_readline (prompt);
+      retval = interactive_input (prompt);
+
+      // There is no need to update the load_path cache if there is no
+      // user input.
+      if (! retval.empty ()
+	  && retval.find_first_not_of (" \t\n\r") != std::string::npos)
+	{
+	  load_path::update ();
+
+	  if (Vdebugging)
+	    last_debugging_command = retval;
+	  else
+	    last_debugging_command = std::string ();
+	}
+      else if (Vdebugging)
+	{
+	  retval = last_debugging_command;
+	  history_skip_auto_repeated_debugging_command = true;
+	}
     }
   else
     retval = gnu_readline ("");
@@ -235,7 +300,8 @@ octave_gets (void)
 
   if (! current_input_line.empty ())
     {
-      if (! (input_from_startup_file || input_from_command_line_file))
+      if (! (input_from_startup_file || input_from_command_line_file
+	     || history_skip_auto_repeated_debugging_command))
 	command_history::add (current_input_line);
 
       if (! (reading_fcn_file || reading_script_file))
@@ -273,7 +339,7 @@ get_user_input (void)
 
 	  size_t len = retval.length ();
 
-	  if (retval[len-1] != '\n')
+	  if (len > 0 && retval[len-1] != '\n')
 	    retval.append ("\n");
 	}
     }
@@ -282,16 +348,13 @@ get_user_input (void)
 
   current_input_line = retval;
 
-  if (! get_input_from_eval_string)
-    input_line_number++;
-
   return retval;
 }
 
 int
 octave_read (char *buf, unsigned max_size)
 {
-  // XXX FIXME XXX -- is this a safe way to buffer the input?
+  // FIXME -- is this a safe way to buffer the input?
 
   static const char * const eol = "\n";
   static std::string input_buf;
@@ -361,7 +424,7 @@ get_input_from_file (const std::string& name, int warn)
   FILE *instream = 0;
 
   if (name.length () > 0)
-    instream = fopen (name.c_str (), "r");
+    instream = fopen (name.c_str (), "rb");
 
   if (! instream && warn)
     warning ("%s: no such file or directory", name.c_str ());
@@ -385,7 +448,7 @@ get_input_from_stdin (void)
   return command_editor::get_input_stream ();
 }
 
-// XXX FIXME XXX -- make this generate file names when appropriate.
+// FIXME -- make this generate file names when appropriate.
 
 static string_vector
 generate_possible_completions (const std::string& text, std::string& prefix,
@@ -402,7 +465,7 @@ generate_possible_completions (const std::string& text, std::string& prefix,
 
   // Sort and remove duplicates.
 
-  names.qsort (true);
+  names.sort (true);
 
   return names;
 }
@@ -467,10 +530,10 @@ generate_completion (const std::string& text, int state)
 	      else
 		retval = name;
 
-	      // XXX FIXME XXX -- looks_like_struct is broken for now,
+	      // FIXME -- looks_like_struct is broken for now,
 	      // so it always returns false.
 
- 	      if (matches == 1 && looks_like_struct (retval))
+	      if (matches == 1 && looks_like_struct (retval))
  		{
  		  // Don't append anything, since we don't know
  		  // whether it should be '(' or '.'.
@@ -489,6 +552,15 @@ generate_completion (const std::string& text, int state)
   return retval;
 }
 
+static std::string
+quoting_filename (const std::string &text, int, char quote)
+{
+  if (quote)
+    return text;
+  else
+    return (std::string ("'") + text);
+}
+
 void
 initialize_command_input (void)
 {
@@ -497,7 +569,7 @@ initialize_command_input (void)
 
   command_editor::set_name ("Octave");
 
-  // XX FIXME XXX -- this needs to include a comma too, but that
+  // FIXME -- this needs to include a comma too, but that
   // causes trouble for the new struct element completion code.
 
   static const char *s = "\t\n !\"\'*+-/:;<=>(){}[\\]^`~";
@@ -508,30 +580,134 @@ initialize_command_input (void)
 
   command_editor::set_basic_quote_characters ("\"");
 
+  command_editor::set_filename_quote_characters (" \t\n\\\"'@<>=;|&()#$`?*[!:{");
+  command_editor::set_completer_quote_characters ("'\"");
+
   command_editor::set_completion_function (generate_completion);
+
+  command_editor::set_quoting_function (quoting_filename);
 }
 
-static bool
-match_sans_spaces_semi (const std::string& standard, const std::string& test)
+static void
+get_debug_input (const std::string& prompt)
 {
-  size_t beg = test.find_first_not_of (" \t");
+  octave_user_code *caller = octave_call_stack::caller_user_code ();
+  std::string nm;
 
-  if (beg != NPOS)
+  int curr_debug_line = tree_evaluator::debug_line ();
+
+  bool have_file = false;
+
+  if (caller)
     {
-      size_t end = test.find_last_not_of ("; \t");
+      nm = caller->fcn_file_name ();
 
-      size_t len = end == NPOS ? NPOS : end - beg + 1;
+      if (nm.empty ())
+	nm = caller->name ();
+      else
+	have_file = true;
+    }
+  else
+    curr_debug_line = -1;
 
-      return (test.substr (beg, len) == standard);
+  std::ostringstream buf;
+
+  if (! nm.empty ())
+    {
+      if (Vgud_mode)
+	{
+	  static char ctrl_z = 'Z' & 0x1f;
+
+	  buf << ctrl_z << ctrl_z << nm << ":" << curr_debug_line;
+	}
+      else
+	{
+	  // FIXME -- we should come up with a clean way to detect
+	  // that we are stopped on the no-op command that marks the
+	  // end of a function or script.
+
+	  buf << "stopped in " << nm;
+
+	  if (curr_debug_line > 0)
+	    buf << " at line " << curr_debug_line;
+
+	  if (have_file)
+	    {
+	      std::string line_buf
+		= get_file_line (nm, curr_debug_line);
+
+	      if (! line_buf.empty ())
+		buf << "\n" << curr_debug_line << ": " << line_buf;
+	    }
+	}
     }
 
-  return false;
+  std::string msg = buf.str ();
+
+  if (! msg.empty ())
+    {
+      if (! Vgud_mode)
+	std::cerr << "keyboard: ";
+
+      std::cerr << msg << std::endl;
+    }
+
+  unwind_protect::begin_frame ("get_debug_input");
+
+  unwind_protect_str (VPS1);
+  VPS1 = prompt;
+
+  while (Vdebugging)
+    {
+      reset_error_handler ();
+
+      reset_parser ();
+
+      // Save current value of global_command.
+      unwind_protect_ptr (global_command);
+
+      // Do this with an unwind-protect cleanup function so that the
+      // forced variables will be unmarked in the event of an interrupt.
+      symbol_table::scope_id scope = symbol_table::top_scope ();
+      unwind_protect::add (symbol_table::unmark_forced_variables, &scope);
+
+      // This is the same as yyparse in parse.y.
+      int retval = octave_parse ();
+
+      if (retval == 0 && global_command)
+	{
+	  global_command->accept (*current_evaluator);
+
+	  // FIXME -- To avoid a memory leak, global_command should be
+	  // deleted, I think.  But doing that here causes trouble if
+	  // an error occurs while executing a debugging command
+	  // (dbstep, for example). It's not clear to me why that
+	  // happens.
+	  //
+	  // delete global_command;
+	  //
+	  // global_command = 0;
+
+	  if (octave_completion_matches_called)
+	    octave_completion_matches_called = false;	    
+	}
+
+      // Unmark forced variables.
+      unwind_protect::run ();
+
+      // Restore previous value of global_command.
+      unwind_protect::run ();
+
+      OCTAVE_QUIT;
+    }
+
+  unwind_protect::run_frame ("get_debug_input");
 }
 
 // If the user simply hits return, this will produce an empty matrix.
 
 static octave_value_list
-get_user_input (const octave_value_list& args, bool debug, int nargout)
+get_user_input (const octave_value_list& args, int nargout)
 {
   octave_value_list retval;
 
@@ -542,64 +718,21 @@ get_user_input (const octave_value_list& args, bool debug, int nargout)
   if (nargin == 2)
     read_as_string++;
 
-  std::string nm;
-  int line = -1;
+  std::string prompt = args(0).string_value ();
 
-  // We look at curr_caller_function because curr_function is always
-  // "keyboard".
-
-  if (debug && curr_caller_function)
+  if (error_state)
     {
-      nm = curr_caller_function->fcn_file_name ();
-
-      if (nm.empty ())
-	nm = curr_caller_function->name ();
-
-      if (curr_statement)
-	line = curr_statement->line ();
+      error ("input: unrecognized argument");
+      return retval;
     }
 
-  OSSTREAM buf;
-
-  if (! nm.empty ())
-    {
-      buf << "stopped in " << nm;
-
-      if (line > 0)
-	buf << " at line " << line;
-    }
-    
-  buf << OSSTREAM_ENDS;
-
-  std::string msg = OSSTREAM_STR (buf);
-
-  OSSTREAM_FREEZE (buf);
-
-  if (! msg.empty ())
-    message ("keyboard", msg.c_str ());
-
-  std::string prompt = "debug> ";
-
-  if (nargin > 0)
-    {
-      prompt = args(0).string_value ();
-
-      if (error_state)
-	{
-	  error ("input: unrecognized argument");
-	  return retval;
-	}
-    }
-
- again:
-
   flush_octave_stdout ();
 
   octave_diary << prompt;
 
-  std::string input_buf = gnu_readline (prompt.c_str (), true);
+  std::string input_buf = interactive_input (prompt.c_str (), true);
 
-  if (! input_buf.empty ())
+  if (! (error_state || input_buf.empty ()))
     {
       if (! input_from_startup_file)
 	command_history::add (input_buf);
@@ -612,47 +745,11 @@ get_user_input (const octave_value_list& args, bool debug, int nargout)
 	octave_diary << "\n";
 
       if (len < 1)
-	{
-	  if (debug)
-	    goto again;
-	  else
-	    return read_as_string ? octave_value ("") : octave_value (Matrix ());
-	}
-
-      if (debug)
-	{
-	  if (match_sans_spaces_semi ("exit", input_buf)
-	      || match_sans_spaces_semi ("quit", input_buf)
-	      || match_sans_spaces_semi ("return", input_buf)
-	      || match_sans_spaces_semi ("dbg_cont", input_buf))
-	    {
-	      return retval;
-	    }
-	  else if (match_sans_spaces_semi ("dbg_step", input_buf))
-	    {
-	      tree::break_next = true;
-
-	      tree::last_line = 0;
-
-	      tree::break_function = curr_function;
-
-	      return retval;
-	    }
-	  else if (match_sans_spaces_semi ("dbg_next", input_buf))
-	    {
-	      tree::break_next = true;
-
-	      tree::last_line = curr_statement->line ();
-
-	      tree::break_function = curr_function;
-
-	      return retval;
-	    }
-	}
+	return read_as_string ? octave_value ("") : octave_value (Matrix ());
 
       if (read_as_string)
 	{
-	  // XXX FIXME XXX -- fix gnu_readline and octave_gets instead!
+	  // FIXME -- fix gnu_readline and octave_gets instead!
 	  if (input_buf.length () == 1 && input_buf[0] == '\n')
 	    retval(0) = "";
 	  else
@@ -662,30 +759,15 @@ get_user_input (const octave_value_list& args, bool debug, int nargout)
 	{
 	  int parse_status = 0;
 
-	  bool silent = ! debug;
+	  retval = eval_string (input_buf, true, parse_status, nargout);
 
-	  retval = eval_string (input_buf, silent, parse_status, nargout);
-
-	  if (! debug && retval.length () == 0)
+	  if (! Vdebugging && retval.length () == 0)
 	    retval(0) = Matrix ();
 	}
     }
   else
     error ("input: reading user-input failed!");
 
-  if (debug)
-    {
-      // Clear error_state so that if errors were encountered while
-      // evaluating user input, extra error messages will not be
-      // printed after we return.
-
-      error_state = 0;
-
-      retval = octave_value_list ();
-
-      goto again;
-    }
-
   return retval;
 }
 
@@ -730,9 +812,64 @@ the screen before your prompt.  @xref{Input and Output}.\n\
   int nargin = args.length ();
 
   if (nargin == 1 || nargin == 2)
-    retval = get_user_input (args, false, nargout);
+    retval = get_user_input (args, nargout);
   else
-    print_usage ("input");
+    print_usage ();
+
+  return retval;
+}
+
+bool
+octave_yes_or_no (const std::string& prompt)
+{
+  std::string prompt_string = prompt + "(yes or no) ";
+
+  while (1)
+    {
+      std::string input_buf = interactive_input (prompt_string, true);
+
+      if (input_buf == "yes")
+	return true;
+      else if (input_buf == "no")
+	return false;
+      else
+	message (0, "Please answer yes or no.");
+    }
+}
+
+DEFUN (yes_or_no, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} yes_or_no (@var{prompt})\n\
+Ask the user a yes-or-no question.  Return 1 if the answer is yes.\n\
+Takes one argument, which is the string to display to ask the\n\
+question.  It should end in a space; @samp{yes-or-no-p} adds\n\
+@samp{(yes or no) } to it.  The user must confirm the answer with\n\
+RET and can edit it until it has been confirmed.\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  int nargin = args.length ();
+
+  if (nargin == 0 || nargin == 1)
+    {
+      std::string prompt;
+
+      if (nargin == 1)
+	{
+	  prompt = args(0).string_value ();
+
+	  if (error_state)
+	    {
+	      error ("yes_or_no: expecting argument to be character string");
+	      return retval;
+	    }
+	}
+
+      retval = octave_yes_or_no (prompt);
+    }
+  else
+    print_usage ();
 
   return retval;
 }
@@ -743,6 +880,14 @@ restore_command_history (void *)
   command_history::ignore_entries (! Vsaving_history);
 }
 
+static size_t saved_frame = 0;
+
+static void
+restore_frame (void *)
+{
+  octave_call_stack::goto_frame (saved_frame);
+}
+
 octave_value
 do_keyboard (const octave_value_list& args)
 {
@@ -754,7 +899,7 @@ do_keyboard (const octave_value_list& args)
 
   unwind_protect::begin_frame ("do_keyboard");
 
-  // XXX FIXME XXX -- we shouldn't need both the
+  // FIXME -- we shouldn't need both the
   // command_history object and the
   // Vsaving_history variable...
   command_history::ignore_entries (false);
@@ -762,12 +907,21 @@ do_keyboard (const octave_value_list& args)
   unwind_protect::add (restore_command_history, 0);
 
   unwind_protect_bool (Vsaving_history);
+  unwind_protect_bool (Vdebugging);
+
+  saved_frame = octave_call_stack::current_frame ();
+  unwind_protect::add (restore_frame);
+  unwind_protect_size_t (saved_frame);
 
   Vsaving_history = true;
+  Vdebugging = true;
 
-  octave_value_list tmp = get_user_input (args, true, 0);
+  std::string prompt = "debug> ";
+  if (nargin > 0)
+    prompt = args(0).string_value ();
 
-  retval = tmp(0);
+  if (! error_state)
+    get_debug_input (prompt);
 
   unwind_protect::run_frame ("do_keyboard");
 
@@ -776,17 +930,19 @@ do_keyboard (const octave_value_list& args)
 
 DEFUN (keyboard, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} keyboard (@var{prompt})\n\
+@deftypefn  {Built-in Function} {} keyboard ()\n\
+@deftypefnx {Built-in Function} {} keyboard (@var{prompt})\n\
 This function is normally used for simple debugging.  When the\n\
 @code{keyboard} function is executed, Octave prints a prompt and waits\n\
 for user input.  The input strings are then evaluated and the results\n\
 are printed.  This makes it possible to examine the values of variables\n\
-within a function, and to assign new values to variables.  No value is\n\
-returned from the @code{keyboard} function, and it continues to prompt\n\
-for input until the user types @samp{quit}, or @samp{exit}.\n\
+within a function, and to assign new values if necessary.  To leave the\n\
+prompt and return to normal execution type @samp{return} or @samp{dbcont}.\n\
+The @code{keyboard} function does not return an exit status.\n\
 \n\
-If @code{keyboard} is invoked without any arguments, a default prompt of\n\
+If @code{keyboard} is invoked without arguments, a default prompt of\n\
 @samp{debug> } is used.\n\
+@seealso{dbcont, dbquit}\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -794,14 +950,25 @@ If @code{keyboard} is invoked without any arguments, a default prompt of\n\
   int nargin = args.length ();
 
   if (nargin == 0 || nargin == 1)
-    do_keyboard (args);
+    {
+      saved_frame = octave_call_stack::current_frame ();
+      unwind_protect::add (restore_frame);
+      unwind_protect_size_t (saved_frame);
+
+      // Skip the frame assigned to the keyboard function.
+      octave_call_stack::goto_frame (1);
+
+      do_keyboard (args);
+
+      unwind_protect::run ();
+    }
   else
-    print_usage ("keyboard");
+    print_usage ();
 
   return retval;
 }
 
-DEFCMD (echo, args, ,
+DEFUN (echo, args, ,
   "-*- texinfo -*-\n\
 @deffn {Command} echo options\n\
 Control whether commands are displayed as they are executed.  Valid\n\
@@ -824,8 +991,7 @@ functions.\n\
 @end table\n\
 \n\
 @noindent\n\
-If invoked without any arguments, @code{echo} toggles the current echo\n\
-state.\n\
+With no arguments, @code{echo} toggles the current echo state.\n\
 @end deffn")
 {
   octave_value_list retval;
@@ -843,9 +1009,9 @@ state.\n\
       {
 	if ((Vecho_executing_commands & ECHO_SCRIPTS)
 	    || (Vecho_executing_commands & ECHO_FUNCTIONS))
-	  bind_builtin_variable ("echo_executing_commands", ECHO_OFF);
+	  Vecho_executing_commands = ECHO_OFF;
 	else
-	  bind_builtin_variable ("echo_executing_commands", ECHO_SCRIPTS);
+	  Vecho_executing_commands = ECHO_SCRIPTS;
       }
       break;
 
@@ -854,11 +1020,11 @@ state.\n\
 	std::string arg = argv[1];
 
 	if (arg == "on")
-	  bind_builtin_variable ("echo_executing_commands", ECHO_SCRIPTS);
+	  Vecho_executing_commands = ECHO_SCRIPTS;
 	else if (arg == "off")
-	  bind_builtin_variable ("echo_executing_commands", ECHO_OFF);
+	  Vecho_executing_commands = ECHO_OFF;
 	else
-	  print_usage ("echo");
+	  print_usage ();
       }
       break;
 
@@ -869,17 +1035,17 @@ state.\n\
 	if (arg == "on" && argv[2] == "all")
 	  {
 	    int tmp = (ECHO_SCRIPTS | ECHO_FUNCTIONS);
-	    bind_builtin_variable ("echo_executing_commands", tmp);
+	    Vecho_executing_commands = tmp;
 	  }
 	else if (arg == "off" && argv[2] == "all")
-	  bind_builtin_variable ("echo_executing_commands", ECHO_OFF);
+	  Vecho_executing_commands = ECHO_OFF;
 	else
-	  print_usage ("echo");
+	  print_usage ();
       }
       break;
 
     default:
-      print_usage ("echo");
+      print_usage ();
       break;
     }
 
@@ -957,7 +1123,7 @@ a feature, not a bug.\n\
 	}
     }
   else
-    print_usage ("completion_matches");
+    print_usage ();
 
   return retval;
 }
@@ -965,9 +1131,12 @@ a feature, not a bug.\n\
 DEFUN (read_readline_init_file, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} read_readline_init_file (@var{file})\n\
-Read the readline library initialiazation file @var{file}.  If\n\
+Read the readline library initialization file @var{file}.  If\n\
 @var{file} is omitted, read the default initialization file (normally\n\
-@file{~/.inputrc}.\n\
+@file{~/.inputrc}).\n\
+\n\
+@xref{Readline Init File, , , readline, GNU Readline Library},\n\
+for details.\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -978,218 +1147,224 @@ Read the readline library initialiazation file @var{file}.  If\n\
     command_editor::read_init_file ();
   else if (nargin == 1)
     {
-      std::string file = file_ops::tilde_expand (args(0).string_value ());
+      std::string file = args(0).string_value ();
 
       if (! error_state)
 	command_editor::read_init_file (file);
     }
   else
-    print_usage ("read_readline_init_file");
+    print_usage ();
 
   return retval;
 }
 
-static std::string hook_fcn;
-static octave_value user_data;
+DEFUN (re_read_readline_init_file, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} re_read_readline_init_file ()\n\
+Re-read the last readline library initialization file that was read.\n\
+@xref{Readline Init File, , , readline, GNU Readline Library},\n\
+for details.\n\
+@end deftypefn")
+{
+  octave_value_list retval;
+
+  if (args.length () == 0)
+    command_editor::re_read_init_file ();
+  else
+    print_usage ();
+
+  return retval;
+}
+
+typedef std::map<std::string, octave_value> hook_fcn_map_type;
+
+static hook_fcn_map_type hook_fcn_map;
 
 static int
 input_event_hook (void)
 {
-  if (is_valid_function (hook_fcn))
-    {
-      if (user_data.is_defined ())
-	feval (hook_fcn, user_data, 0);
-      else
-	feval (hook_fcn, octave_value_list (), 0);
-    }
-  else
-    {
-      hook_fcn = std::string ();
-      user_data = octave_value ();
+  hook_fcn_map_type::iterator p = hook_fcn_map.begin ();
 
-      command_editor::set_event_hook (0);
+  while (p != hook_fcn_map.end ())
+    {
+      std::string hook_fcn = p->first;
+      octave_value user_data = p->second;
+
+      p++;
+
+      if (is_valid_function (hook_fcn))
+	{
+	  if (user_data.is_defined ())
+	    feval (hook_fcn, user_data, 0);
+	  else
+	    feval (hook_fcn, octave_value_list (), 0);
+	}
+      else
+	hook_fcn_map.erase (p);
     }
 
+  if (hook_fcn_map.empty ())
+    command_editor::remove_event_hook (input_event_hook);
+
   return 0;
 }
 
-DEFUN (input_event_hook, args, ,
+DEFUN (add_input_event_hook, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {[@var{ofcn}, @var{odata}] =} input_event_hook (@var{fcn}, @var{data})\n\
-Given the name of a function as a string and any Octave value object,\n\
-install @var{fcn} as a function to call periodically, when Octave is\n\
-waiting for input.  The function should have the form\n\
+@deftypefn {Built-in Function} {} add_input_event_hook (@var{fcn}, @var{data})\n\
+Add the named function @var{fcn} to the list of functions to call\n\
+periodically when Octave is waiting for input.  The function should\n\
+have the form\n\
 @example\n\
 @var{fcn} (@var{data})\n\
 @end example\n\
 \n\
 If @var{data} is omitted, Octave calls the function without any\n\
-arguments.  If both @var{fcn} and @var{data} are omitted, Octave\n\
-clears the hook.  In all cases, the name of the previous hook function\n\
-and the user data are returned.\n\
+arguments.\n\
+@seealso{remove_input_event_hook}\n\
 @end deftypefn")
 {
   octave_value_list retval;
 
   int nargin = args.length ();
 
-  if (nargin > 2)
-    print_usage ("input_event_hook");
-  else
+  if (nargin == 1 || nargin == 2)
     {
-      octave_value tmp_user_data;
+      octave_value user_data;
 
-      std::string tmp_hook_fcn;
+      if (nargin == 2)
+	user_data = args(1);
 
-      if (nargin > 1)
-	tmp_user_data = args(1);
+      std::string hook_fcn = args(0).string_value ();
 
-      if (nargin > 0)
+      if (! error_state)
 	{
-	  tmp_hook_fcn = args(0).string_value ();
+	  if (hook_fcn_map.empty ())
+	    command_editor::add_event_hook (input_event_hook);
 
-	  if (error_state)
-	    {
-	      error ("input_event_hook: expecting string as first arg");
-	      return retval;
-	    }
-
-	  command_editor::set_event_hook (input_event_hook);
+	  hook_fcn_map[hook_fcn] = user_data;
 	}
-
-      if (nargin == 0)
-	command_editor::set_event_hook (0);
-
-      retval(1) = user_data;
-      retval(0) = hook_fcn;
-
-      hook_fcn = tmp_hook_fcn;
-      user_data = tmp_user_data;
+      else
+	error ("add_input_event_hook: expecting string as first arg");
     }
+  else
+    print_usage ();
 
   return retval;
 }
 
-static int
-ps1 (void)
+DEFUN (remove_input_event_hook, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} remove_input_event_hook (@var{fcn})\n\
+Remove the named function @var{fcn} to the list of functions to call\n\
+periodically when Octave is waiting for input.\n\
+@seealso{add_input_event_hook}\n\
+@end deftypefn")
 {
-  int status = 0;
+  octave_value_list retval;
 
-  Vps1 = builtin_string_variable ("PS1");
+  int nargin = args.length ();
 
-  return status;
-}
-
-static int
-ps2 (void)
-{
-  int status = 0;
-
-  Vps2 = builtin_string_variable ("PS2");
-
-  return status;
-}
-
-static int
-ps4 (void)
-{
-  int status = 0;
-
-  Vps4 = builtin_string_variable ("PS4");
-
-  return status;
-}
-
-static int
-completion_append_char (void)
-{
-  int status = 0;
-
-  std::string s = builtin_string_variable ("completion_append_char");
-
-  switch (s.length ())
+  if (nargin == 1)
     {
-    case 1:
-      Vcompletion_append_char = s[0];
-      break;
+      std::string hook_fcn = args(0).string_value ();
 
-    case 0:
-      Vcompletion_append_char = '\0';
-      break;
+      if (! error_state)
+	{
+	  hook_fcn_map_type::iterator p = hook_fcn_map.find (hook_fcn);
 
-    default:
-      warning ("completion_append_char must be a single character");
-      status = -1;
-      break;
+	  if (p != hook_fcn_map.end ())
+	    hook_fcn_map.erase (p);
+	  else
+	    error ("remove_input_event_hook: %s not found in list",
+		   hook_fcn.c_str ());
+
+	  if (hook_fcn_map.empty ())
+	    command_editor::remove_event_hook (input_event_hook);
+	}
+      else
+	error ("remove_input_event_hook: expecting string as first arg");
     }
+  else
+    print_usage ();
 
-  return status;
+  return retval;
 }
 
-static int
-echo_executing_commands (void)
-{
-  Vecho_executing_commands = check_preference ("echo_executing_commands"); 
-
-  return 0;
-}
-
-void
-symbols_of_input (void)
-{
-  DEFVAR (PS1, "\\s:\\#> ", ps1,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} PS1\n\
-The primary prompt string.  When executing interactively, Octave\n\
-displays the primary prompt @code{PS1} when it is ready to read a\n\
-command.\n\
+DEFUN (PS1, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} PS1 ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} PS1 (@var{new_val})\n\
+Query or set the primary prompt string.  When executing interactively,\n\
+Octave displays the primary prompt when it is ready to read a command.\n\
 \n\
-The default value of @code{PS1} is @code{\"\\s:\\#> \"}.  To change it, use a\n\
-command like\n\
+The default value of the primary prompt string is @code{\"\\s:\\#> \"}.\n\
+To change it, use a command like\n\
 \n\
 @example\n\
-octave:13> PS1 = \"\\\\u@@\\\\H> \"\n\
+octave:13> PS1 (\"\\\\u@@\\\\H> \")\n\
 @end example\n\
 \n\
 @noindent\n\
 which will result in the prompt @samp{boris@@kremvax> } for the user\n\
 @samp{boris} logged in on the host @samp{kremvax.kgb.su}.  Note that two\n\
-backslashes are required to enter a backslash into a string.\n\
+backslashes are required to enter a backslash into a double-quoted\n\
+character string.\n\
 @xref{Strings}.\n\
-@end defvr");
+@seealso{PS2, PS4}\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (PS1);
+}
 
-  DEFVAR (PS2, "> ", ps2,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} PS2\n\
-The secondary prompt string, which is printed when Octave is\n\
-expecting additional input to complete a command.  For example, when\n\
-defining a function over several lines, Octave will print the value of\n\
-@code{PS1} at the beginning of each line after the first.  The default\n\
-value of @code{PS2} is @code{\"> \"}.\n\
-@end defvr");
+DEFUN (PS2, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} PS2 ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} PS2 (@var{new_val})\n\
+Query or set the secondary prompt string.  The secondary prompt is\n\
+printed when Octave is expecting additional input to complete a\n\
+command.  For example, if you are typing a @code{for} loop that spans several\n\
+lines, Octave will print the secondary prompt at the beginning of\n\
+each line after the first.  The default value of the secondary prompt\n\
+string is @code{\"> \"}.\n\
+@seealso{PS1, PS4}\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (PS2);
+}
 
-  DEFVAR (PS4, "+ ", ps4,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} PS4\n\
-If Octave is invoked with the @code{--echo-commands} option, the value of\n\
-@code{PS4} is printed before each line of input that is echoed.  The\n\
-default value of @code{PS4} is @code{\"+ \"}.  @xref{Invoking Octave}, for\n\
-a description of @code{--echo-commands}.\n\
-@end defvr");
+DEFUN (PS4, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} PS4 ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} PS4 (@var{new_val})\n\
+Query or set the character string used to prefix output produced\n\
+when echoing commands is enabled.\n\
+The default value is @code{\"+ \"}.\n\
+@xref{Diary and Echo Commands}, for a description of echoing commands.\n\
+@seealso{echo, echo_executing_commands, PS1, PS2}\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (PS4);
+}
 
-  DEFVAR (completion_append_char, " ", completion_append_char,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} completion_append_char\n\
-The value of @code{completion_append_char} is used as the character to\n\
-append to successful command-line completion attempts.  The default\n\
+DEFUN (completion_append_char, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} completion_append_char ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} completion_append_char (@var{new_val})\n\
+Query or set the internal character variable that is appended to\n\
+successful command-line completion attempts.  The default\n\
 value is @code{\" \"} (a single space).\n\
-@end defvr");
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (completion_append_char);
+}
 
-  DEFVAR (echo_executing_commands, ECHO_OFF, echo_executing_commands,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} echo_executing_commands\n\
-This variable may also be used to control the echo state.  It may be\n\
-the sum of the following values:\n\
+DEFUN (echo_executing_commands, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} echo_executing_commands ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} echo_executing_commands (@var{new_val})\n\
+Query or set the internal variable that controls the echo state.\n\
+It may be the sum of the following values:\n\
 \n\
 @table @asis\n\
 @item 1\n\
@@ -1205,9 +1380,90 @@ Echo commands read from command line.\n\
 More than one state can be active at once.  For example, a value of 3 is\n\
 equivalent to the command @kbd{echo on all}.\n\
 \n\
-The value of @code{echo_executing_commands} is set by the @kbd{echo}\n\
-command and the command line option @code{--echo-input}.\n\
-@end defvr");
+The value of @code{echo_executing_commands} may be set by the @kbd{echo}\n\
+command or the command line option @code{--echo-commands}.\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (echo_executing_commands);
+}
+
+DEFUN (__request_drawnow__, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} __request_drawnow__ ()\n\
+@deftypefnx {Built-in Function} {} __request_drawnow__ (@var{flag})\n\
+Undocumented internal function.\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  int nargin = args.length ();
+
+  if (nargin == 0)
+    Vdrawnow_requested = true;
+  else if (nargin == 1)
+    Vdrawnow_requested = args(0).bool_value ();
+  else
+    print_usage ();
+
+  return retval;
+}
+
+DEFUN (__gud_mode__, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} __gud_mode__ ()\n\
+Undocumented internal function.\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  int nargin = args.length ();
+
+  if (nargin == 0)
+    retval = Vgud_mode;
+  else if (nargin == 1)
+    Vgud_mode = args(0).bool_value ();
+  else
+    print_usage ();
+
+  return retval;
+}
+
+DEFUN (filemarker, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} filemarker ()\n\
+Returns or sets the character used to separate filename from the\n\
+the subfunction names contained within the file.  This can be used in\n\
+a generic manner to interact with subfunctions.  For example\n\
+\n\
+@example\n\
+help ([\"myfunc\", filemarker, \"mysubfunc\"])\n\
+@end example\n\
+\n\
+@noindent\n\
+returns the help string associated with the sub-function @code{mysubfunc}\n\
+of the function @code{myfunc}.  Another use of @code{filemarker} is when\n\
+debugging it allows easier placement of breakpoints within sub-functions.\n\
+For example\n\
+\n\
+@example\n\
+dbstop ([\"myfunc\", filemarker, \"mysubfunc\"])\n\
+@end example\n\
+\n\
+@noindent\n\
+will set a breakpoint at the first line of the subfunction @code{mysubfunc}.\n\
+@end deftypefn")
+{
+  char tmp = Vfilemarker;
+  octave_value retval = SET_INTERNAL_VARIABLE (filemarker);
+
+  // The character passed must not be a legal character for a function name
+  if (! error_state && (::isalnum (Vfilemarker) || Vfilemarker == '_'))
+    {
+      Vfilemarker = tmp;
+      error ("filemarker: character can not be a valid character for a function name");
+    }
+
+  return retval;
 }
 
 /*
