@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2005, 2006, 2007
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2005, 2006, 2007, 2008
               John W. Eaton
 
 This file is part of Octave.
@@ -28,7 +28,7 @@ along with Octave; see the file COPYING.  If not, see
 
 #include "cmd-hist.h"
 
-extern void initialize_history (void);
+extern void initialize_history (bool read_history_file = false);
 
 // Write timestamp to history file.
 extern void octave_history_write_timestamp (void);
