@@ -394,15 +394,12 @@ execute_eval_option_code (const std::string& code)
     {
       eval_string (code, false, parse_status, 0);
     }
-  catch (octave_quit_exception e)
-    {
-      unwind_protect::run_frame ("execute_eval_option_code");
-      clean_up_and_exit (e.status);
-    }
   catch (octave_interrupt_exception)
     {
       recover_from_exception ();
       octave_stdout << "\n";
+      if (quitting_gracefully)
+        clean_up_and_exit (exit_status);
     }
   catch (std::bad_alloc)
     {
@@ -466,15 +463,12 @@ execute_command_line_file (const std::string& fname)
 
       source_file (fname, context, verbose, require_file, "octave");
     }
-  catch (octave_quit_exception e)
-    {
-      unwind_protect::run_frame ("execute_command_line_file");
-      clean_up_and_exit (e.status);
-    }
   catch (octave_interrupt_exception)
     {
       recover_from_exception ();
       octave_stdout << "\n";
+      if (quitting_gracefully)
+        clean_up_and_exit (exit_status);
     }
   catch (std::bad_alloc)
     {
@@ -777,10 +771,24 @@ octave_main (int argc, char **argv, int embedded)
   atexit (do_octave_atexit);
 #endif
 
+  // Is input coming from a terminal?  If so, we are probably
+  // interactive.
+
+  interactive = (! embedded
+		 && isatty (fileno (stdin)) && isatty (fileno (stdout)));
+
+  if (! interactive && ! forced_line_editing)
+    line_editing = false;
+
+  // Force default line editor if we don't want readline editing.
+  if (! line_editing)
+    command_editor::force_default_editor ();
+
   // These can come after command line args since none of them set any
   // defaults that might be changed by command line options.
 
-  initialize_command_input ();
+  if (line_editing)
+    initialize_command_input ();
 
   if (! inhibit_startup_message)
     std::cout << OCTAVE_STARTUP_MESSAGE "\n" << std::endl;
@@ -806,15 +814,6 @@ octave_main (int argc, char **argv, int embedded)
   if (! inhibit_startup_message && reading_startup_message_printed)
     std::cout << std::endl;
 
-  // Is input coming from a terminal?  If so, we are probably
-  // interactive.
-
-  interactive = (! embedded
-		 && isatty (fileno (stdin)) && isatty (fileno (stdout)));
-
-  if (! interactive && ! forced_line_editing)
-    line_editing = false;
-
   // If there is an extra argument, see if it names a file to read.
   // Additional arguments are taken as command line options for the
   // script.
