@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2005,
-              2006, 2007 John W. Eaton
+              2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,6 +26,7 @@ along with Octave; see the file COPYING.  If not, see
 #endif
 
 #include "error.h"
+#include "gripes.h"
 #include "oct-obj.h"
 #include "ov-builtin.h"
 #include "ov.h"
@@ -101,7 +102,17 @@ octave_builtin::do_multi_index_op (int nargout, const octave_value_list& args)
 
       unwind_protect::add (octave_call_stack::unwind_pop, 0);
 
-      retval = (*f) (args, nargout);
+      try
+	{
+	  retval = (*f) (args, nargout);
+          // Do not allow null values to be returned from functions.
+          // FIXME -- perhaps true builtins should be allowed?
+          retval.make_storable_values ();
+	}
+      catch (octave_execution_exception)
+	{
+	  gripe_library_execution_error ();
+	}
 
       unwind_protect::run_frame ("builtin_func_eval");
     }
