@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005, 2006, 2007
+Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
               John W. Eaton
 
 This file is part of Octave.
@@ -29,11 +29,13 @@ along with Octave; see the file COPYING.  If not, see
 #include <iostream>
 #include <sstream>
 #include <vector>
+#include <queue>
 
 #include "Array-util.h"
 #include "byte-swap.h"
 #include "lo-utils.h"
 #include "quit.h"
+#include "oct-locbuf.h"
 
 #include "defun.h"
 #include "error.h"
@@ -47,12 +49,49 @@ along with Octave; see the file COPYING.  If not, see
 #include "ov-scalar.h"
 #include "pr-output.h"
 #include "ov-scalar.h"
+#include "gripes.h"
 
 #include "ls-oct-ascii.h"
 #include "ls-oct-binary.h"
 #include "ls-hdf5.h"
 #include "ls-utils.h"
 
+// Cell is able to handle octave_value indexing by itself, so just forward
+// everything.
+
+template <>
+octave_value
+octave_base_matrix<Cell>::do_index_op (const octave_value_list& idx,
+                                       bool resize_ok)
+{
+  return matrix.index (idx, resize_ok);
+}
+
+template <>
+void
+octave_base_matrix<Cell>::assign (const octave_value_list& idx, const Cell& rhs)
+{
+  matrix.assign (idx, rhs);
+}
+
+template <>
+void
+octave_base_matrix<Cell>::assign (const octave_value_list& idx, octave_value rhs)
+{
+  // FIXME: Really?
+  if (rhs.is_cell ())
+    matrix.assign (idx, rhs.cell_value ());
+  else
+    matrix.assign (idx, Cell (rhs));
+}
+
+template <>
+void
+octave_base_matrix<Cell>::delete_elements (const octave_value_list& idx)
+{
+  matrix.delete_elements (idx);
+}
+
 template class octave_base_matrix<Cell>;
 
 DEFINE_OCTAVE_ALLOCATOR (octave_cell);
@@ -67,7 +106,8 @@ gripe_failed_assignment (void)
 
 octave_value_list
 octave_cell::subsref (const std::string& type,
-		      const std::list<octave_value_list>& idx, int nargout)
+		      const std::list<octave_value_list>& idx,
+		      int nargout)
 {
   octave_value_list retval;
 
@@ -88,19 +128,7 @@ octave_cell::subsref (const std::string& type,
 	    if (tcell.length () == 1)
 	      retval(0) = tcell(0,0);
 	    else
-	      {
-		octave_idx_type n = tcell.numel ();
-
-		octave_value_list lst (n, octave_value ());
-
-		for (octave_idx_type i = 0; i < n; i++)
-		  {
-		    OCTAVE_QUIT;
-		    lst(i) = tcell(i);
-		  }
-
-		retval(0) = octave_value (lst, true);
-	      }
+              retval = octave_value (octave_value_list (tcell), true);
 	  }
       }
       break;
@@ -126,6 +154,56 @@ octave_cell::subsref (const std::string& type,
   return retval;
 }
 
+octave_value
+octave_cell::subsref (const std::string& type,
+		      const std::list<octave_value_list>& idx,
+		      bool auto_add)
+{
+  octave_value retval;
+
+  switch (type[0])
+    {
+    case '(':
+      retval = do_index_op (idx.front (), auto_add);
+      break;
+
+    case '{':
+      {
+	octave_value tmp = do_index_op (idx.front (), auto_add);
+
+	if (! error_state)
+	  {
+	    const Cell tcell = tmp.cell_value ();
+
+	    if (tcell.length () == 1)
+	      retval = tcell(0,0);
+	    else
+              retval = octave_value (octave_value_list (tcell), true);
+	  }
+      }
+      break;
+
+    case '.':
+      {
+	std::string nm = type_name ();
+	error ("%s cannot be indexed with %c", nm.c_str (), type[0]);
+      }
+      break;
+
+    default:
+      panic_impossible ();
+    }
+
+  // FIXME -- perhaps there should be an
+  // octave_value_list::next_subsref member function?  See also
+  // octave_user_function::subsref.
+
+  if (idx.size () > 1)
+    retval = retval.next_subsref (auto_add, type, idx);
+
+  return retval;
+}
+
 octave_value
 octave_cell::subsasgn (const std::string& type,
 		       const std::list<octave_value_list>& idx,
@@ -137,6 +215,14 @@ octave_cell::subsasgn (const std::string& type,
 
   octave_value t_rhs = rhs;
 
+  clear_cellstr_cache ();
+
+  if (idx.front ().empty ())
+    {
+      error ("missing index in indexed assignment");
+      return retval;
+    }
+
   if (n > 1)
     {
       switch (type[0])
@@ -177,31 +263,36 @@ octave_cell::subsasgn (const std::string& type,
 
 	case '{':
 	  {
-	    octave_value tmp = do_index_op (idx.front (), true);
-
-	    if (! tmp.is_defined ())
-	      tmp = octave_value::empty_conv (type.substr (1), rhs);
+            matrix.make_unique ();
+	    Cell tmpc = matrix.index (idx.front (), true);
 
 	    if (! error_state)
 	      {
-		const Cell tcell = tmp.cell_value ();
+                std::list<octave_value_list> next_idx (idx);
 
-		if (! error_state && tcell.length () == 1)
+                next_idx.erase (next_idx.begin ());
+
+                std::string next_type = type.substr (1);
+
+                if (tmpc.numel () == 1)
 		  {
-		    tmp = tcell(0,0);
-
-		    std::list<octave_value_list> next_idx (idx);
-
-		    next_idx.erase (next_idx.begin ());
-
-		    tmp.make_unique ();
+		    octave_value tmp = tmpc(0);
+                    tmpc = Cell ();
 
 		    if (! tmp.is_defined () || tmp.is_zero_by_zero ())
-		      tmp = octave_value::empty_conv (type.substr (1), rhs);
+                      {
+                        tmp = octave_value::empty_conv (type.substr (1), rhs);
+                        tmp.make_unique (); // probably a no-op.
+                      }
+                    else
+                      // optimization: ignore the copy still stored inside our array. 
+                      tmp.make_unique (1);
 
 		    if (! error_state)
-		      t_rhs = tmp.subsasgn (type.substr (1), next_idx, rhs);
+		      t_rhs = tmp.subsasgn (next_type, next_idx, rhs);
 		  }
+                else
+                  gripe_indexed_cs_list ();
 	      }
 	  }
 	  break;
@@ -229,8 +320,8 @@ octave_cell::subsasgn (const std::string& type,
 	    if (t_rhs.is_cell ())
 	      octave_base_matrix<Cell>::assign (i, t_rhs.cell_value ());
 	    else
-	      if (t_rhs.is_empty ())
-		octave_base_matrix<Cell>::assign (i, Cell());
+	      if (t_rhs.is_null_value ())
+		octave_base_matrix<Cell>::delete_elements (i);
 	      else
 		octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
 
@@ -246,23 +337,29 @@ octave_cell::subsasgn (const std::string& type,
 
 	case '{':
 	  {
-	    octave_value_list i = idx.front ();
+	    octave_value_list idxf = idx.front ();
 
 	    if (t_rhs.is_cs_list ())
 	      {
 		Cell tmp_cell = Cell (t_rhs.list_value ());
 
-		// The shape of the RHS is irrelevant, we just want
-		// the number of elements to agree and to preserve the
-		// shape of the left hand side of the assignment.
+                // Inquire the proper shape of the RHS.
 
-		if (numel () == tmp_cell.numel ())
-		  tmp_cell = tmp_cell.reshape (dims ());
+                dim_vector didx = dims ().redim (idxf.length ());
+                for (octave_idx_type k = 0; k < idxf.length (); k++)
+                  if (! idxf(k).is_magic_colon ()) didx(k) = idxf(k).numel ();
 
-		octave_base_matrix<Cell>::assign (i, tmp_cell);
+                if (didx.numel () == tmp_cell.numel ())
+                  tmp_cell = tmp_cell.reshape (didx);
+
+
+		octave_base_matrix<Cell>::assign (idxf, tmp_cell);
 	      }
-	    else
-	      octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
+	    else if (idxf.all_scalars () || do_index_op (idxf, true).numel () == 1)
+              // Regularize a null matrix if stored into a cell.
+              octave_base_matrix<Cell>::assign (idxf, Cell (t_rhs.storable_value ()));
+            else if (! error_state)
+              error ("invalid assignment to cs-list outside multiple assignment.");
 
 	    if (! error_state)
 	      {
@@ -289,13 +386,63 @@ octave_cell::subsasgn (const std::string& type,
   return retval;
 }
 
-void
+void 
+octave_cell::clear_cellstr_cache (void) const
+{
+  cellstr_cache = Array<std::string> ();
+}
+
+void 
+octave_cell::make_cellstr_cache (void) const
+{
+  cellstr_cache = Array<std::string> (matrix.dims ());
+
+  octave_idx_type n = numel ();
+
+  std::string *dst = cellstr_cache.fortran_vec ();
+  const octave_value *src = matrix.data ();
+
+  for (octave_idx_type i = 0; i < n; i++)
+    dst[i] = src[i].string_value ();
+}
+
+bool 
+octave_cell::is_cellstr (void) const
+{
+  bool retval;
+  if (! cellstr_cache.is_empty ())
+    retval = true;
+  else
+    {
+      retval = matrix.is_cellstr ();
+      // force cache to be created here
+      if (retval)
+        make_cellstr_cache ();
+    }
+
+  return retval;
+}
+
+void 
+octave_cell::assign (const octave_value_list& idx, const Cell& rhs)
+{
+  clear_cellstr_cache ();
+  octave_base_matrix<Cell>::assign (idx, rhs);
+}
+
+void 
 octave_cell::assign (const octave_value_list& idx, const octave_value& rhs)
 {
-  if (rhs.is_cell ())
-    octave_base_matrix<Cell>::assign (idx, rhs.cell_value ());
-  else
-    octave_base_matrix<Cell>::assign (idx, Cell (rhs));
+  clear_cellstr_cache ();
+  octave_base_matrix<Cell>::assign (idx, rhs);
+}
+
+
+void 
+octave_cell::delete_elements (const octave_value_list& idx)
+{
+  clear_cellstr_cache ();
+  octave_base_matrix<Cell>::delete_elements (idx);
 }
 
 size_t
@@ -309,32 +456,110 @@ octave_cell::byte_size (void) const
   return retval;
 }
 
+octave_value
+octave_cell::sort (octave_idx_type dim, sortmode mode) const
+{
+  octave_value retval;
+
+  if (is_cellstr ())
+    {
+      Array<std::string> tmp = cellstr_value ();
+
+      tmp = tmp.sort (dim, mode);
+
+      // We already have the cache.
+      retval = new octave_cell (tmp);
+    }
+  else
+    error ("sort: only cell arrays of character strings may be sorted");
+  
+  return retval;
+}
+
+octave_value
+octave_cell::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
+		   sortmode mode) const
+{
+  octave_value retval;
+
+  if (is_cellstr ())
+    {
+      Array<std::string> tmp = cellstr_value ();
+
+      tmp = tmp.sort (sidx, dim, mode);
+
+      // We already have the cache.
+      retval = new octave_cell (tmp);
+    }
+  else
+    error ("sort: only cell arrays of character strings may be sorted");
+  
+  return retval;
+}
+
+sortmode 
+octave_cell::is_sorted (sortmode mode) const
+{
+  sortmode retval = UNSORTED;
+
+  if (is_cellstr ())
+    {
+      Array<std::string> tmp = cellstr_value ();
+
+      retval = tmp.is_sorted (mode);
+    }
+  else
+    error ("issorted: not a cell array of strings");
+  
+  return retval;
+}
+
+
+Array<octave_idx_type>
+octave_cell::sort_rows_idx (sortmode mode) const
+{
+  Array<octave_idx_type> retval;
+
+  if (is_cellstr ())
+    {
+      Array<std::string> tmp = cellstr_value ();
+
+      retval = tmp.sort_rows_idx (mode);
+    }
+  else
+    error ("sortrows: only cell arrays of character strings may be sorted");
+  
+  return retval;
+}
+
+sortmode 
+octave_cell::is_sorted_rows (sortmode mode) const
+{
+  sortmode retval = UNSORTED;
+
+  if (is_cellstr ())
+    {
+      Array<std::string> tmp = cellstr_value ();
+
+      retval = tmp.is_sorted_rows (mode);
+    }
+  else
+    error ("issorted: not a cell array of strings");
+  
+  return retval;
+}
+
+bool
+octave_cell::is_true (void) const
+{
+  error ("invalid conversion from cell array to logical value");
+  return false;
+}
+
 octave_value_list
 octave_cell::list_value (void) const
 {
-  octave_value_list retval;
-
-  octave_idx_type nr = rows ();
-  octave_idx_type nc = columns ();
-
-  if (nr == 1 && nc > 0)
-    {
-      retval.resize (nc);
-
-      for (octave_idx_type i = 0; i < nc; i++)
-	retval(i) = matrix(0,i);
-    }
-  else if (nc == 1 && nr > 0)
-    {
-      retval.resize (nr);
-
-      for (octave_idx_type i = 0; i < nr; i++)
-	retval(i) = matrix(i,0);
-    }
-  else
-    error ("invalid conversion from cell array to list");
-
-  return retval;
+  return octave_value_list (matrix);
 }
 
 string_vector
@@ -348,6 +573,8 @@ octave_cell::all_strings (bool pad) const
 
   octave_idx_type max_len = 0;
 
+  std::queue<string_vector> strvec_queue;
+
   for (octave_idx_type i = 0; i < nel; i++)
     {
       string_vector s = matrix(i).all_strings ();
@@ -363,15 +590,18 @@ octave_cell::all_strings (bool pad) const
 
       if (s_max_len > max_len)
 	max_len = s_max_len;
+
+      strvec_queue.push (s);
     }
 
-  retval.resize (n_elts);
+  retval = string_vector (n_elts);
 
   octave_idx_type k = 0;
 
   for (octave_idx_type i = 0; i < nel; i++)
     {
-      string_vector s = matrix(i).all_strings ();
+      const string_vector s = strvec_queue.front ();
+      strvec_queue.pop ();
 
       octave_idx_type s_len = s.length ();
 
@@ -397,6 +627,21 @@ octave_cell::all_strings (bool pad) const
   return retval;
 }
 
+Array<std::string>
+octave_cell::cellstr_value (void) const
+{
+  Array<std::string> retval;
+
+  if (is_cellstr ())
+    {
+      retval = cellstr_cache;
+    }
+  else
+    error ("invalid conversion from cell array to array of strings");
+
+  return retval;
+}
+
 bool
 octave_cell::print_as_scalar (void) const
 {
@@ -526,6 +771,8 @@ octave_cell::load_ascii (std::istream& is)
 {
   bool success = true;
 
+  clear_cellstr_cache ();
+
   string_vector keywords(2);
 
   keywords[0] = "ndims";
@@ -690,8 +937,10 @@ octave_cell::save_binary (std::ostream& os, bool& save_as_floats)
 
 bool 
 octave_cell::load_binary (std::istream& is, bool swap,
-				 oct_mach_info::float_format fmt)
+                          oct_mach_info::float_format fmt)
 {
+  clear_cellstr_cache ();
+
   bool success = true;
   int32_t mdims;
   if (! is.read (reinterpret_cast<char *> (&mdims), 4))
@@ -764,6 +1013,13 @@ octave_cell::load_binary (std::istream& is, bool swap,
   return success;
 }
 
+void *
+octave_cell::mex_get_data (void) const
+{
+  clear_cellstr_cache ();
+  return matrix.mex_get_data ();
+}
+
 #if defined (HAVE_HDF5)
 
 bool
@@ -829,7 +1085,7 @@ octave_cell::save_hdf5 (hid_t loc_id, const char *name, bool save_as_floats)
   for (octave_idx_type i = 0; i < nel; i++)
     {
       std::ostringstream buf;
-      int digits = static_cast<int> (floor (log10 (static_cast<double> (nel)) + 1.0));
+      int digits = static_cast<int> (::floor (::log10 (static_cast<double> (nel)) + 1.0));
       buf << "_" << std::setw (digits) << std::setfill ('0') << i;
       std::string s = buf.str ();
 
@@ -850,6 +1106,8 @@ bool
 octave_cell::load_hdf5 (hid_t loc_id, const char *name,
 			bool have_h5giterate_bug)
 {
+  clear_cellstr_cache ();
+
   bool retval = false;
 
   dim_vector dv;
@@ -1015,23 +1273,12 @@ rows and columns, respectively.\n\
 
   if (! error_state)
     {
-      int ndim = dims.length ();
+      dims.chop_trailing_singletons ();
 
       check_dimensions (dims, "cell");
 
       if (! error_state)
-	{
-	  switch (ndim)
-	    {
-	    case 1:
-	      retval = Cell (dims(0), dims(0), Matrix ());
-	      break;
-
-	    default:
-	      retval = Cell (dims, Matrix ());
-	      break;
-	    }
-	}
+        retval = Cell (dims, Matrix ());
     }
 
   return retval;
@@ -1079,7 +1326,9 @@ array @var{string}.\n\
 	  string_vector s = args(0).all_strings ();
 
 	  if (! error_state)
-	    retval = Cell (s, true);
+	    retval = (s.is_empty ()
+                      ? Cell (octave_value (std::string ()))
+                      : Cell (s, true));
 	  else
 	    error ("cellstr: expecting argument to be a 2-d character array");
 	}
@@ -1114,52 +1363,38 @@ cell array will have a dimension vector corresponding to\n\
 
 	  string_vector keys = m.keys ();
 
-	  octave_idx_type fields_numel = keys.length ();
+	  octave_idx_type num_fields = keys.length ();
 
 	  // The resulting dim_vector should have dimensions:
 	  // [numel(fields) size(struct)]
+	  // except if the struct is a column vector.
 
 	  dim_vector result_dv;
-	  result_dv.resize (m_dv.length () + 1); // Add 1 for the fields.
+	  if (m_dv (m_dv.length () - 1) == 1)
+	      result_dv.resize (m_dv.length ());
+	  else
+	      result_dv.resize (m_dv.length () + 1); // Add 1 for the fields.
 
-	  result_dv(0) = fields_numel;
+	  result_dv(0) = num_fields;
 
 	  for (int i = 1; i < result_dv.length (); i++)
 	    result_dv(i) = m_dv(i-1);
 
-	  // Squeeze to be sure that a (3,1) vector doesn't get turned
-	  // into a (3,3,1) vector.
-
-	  result_dv = result_dv.squeeze ();
-
 	  Cell c (result_dv);
 
-	  // Use ra_idx both for counting and for assignments, so
-	  // ra_idx(0) will both contain fields_numel for each call to
-	  // increment_index and j for each assignment.
+	  octave_idx_type n_elts = m.numel ();
 
-	  Array<octave_idx_type> ra_idx (result_dv.length (), 0);
-	  ra_idx(0) = fields_numel;
-
-	  for (octave_idx_type i = 0; i < m_dv.numel (); i++)
+	  for (octave_idx_type j = 0; j < num_fields; j++)
 	    {
-	      for (octave_idx_type j = 0; j < fields_numel; j++)
+	      octave_idx_type k = j;
+
+	      const Cell vals = m.contents (keys(j));
+
+	      for (octave_idx_type i = 0; i < n_elts; i++)
 		{
-		  ra_idx(0) = j;
-
-		  Cell c_tmp = m.contents (keys(j));
-
-		  if (c_tmp.length () > 1) // Is a cell.
-		    c(ra_idx) = c_tmp;
-		  else if (c_tmp.length () == 1) // Get octave_value.
-		    c(ra_idx) = c_tmp(0);
-		  else
-		    c(ra_idx) = Cell ();
-
-		  ra_idx(0) = fields_numel;
+		  c(k) = vals(i);
+		  k += num_fields;
 		}
-
-	      increment_index (ra_idx, result_dv);
 	    }
 
 	  retval = c;
