@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2001, 2002,
-              2004, 2005, 2006, 2007 John W. Eaton
+              2004, 2005, 2006, 2007, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -24,7 +24,7 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_procstream_h)
 #define octave_procstream_h 1
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 #ifdef HAVE_SYS_TYPES_H
