@@ -904,11 +904,20 @@ tree_evaluator::visit_try_catch_command (tree_try_catch_command& cmd)
     }
 }
 
+void restore_interrupt_state (void *ptr)
+{
+  octave_interrupt_state = *(reinterpret_cast<sig_atomic_t *> (ptr));
+}
+
 static void
 do_unwind_protect_cleanup_code (void *ptr)
 {
   tree_statement_list *list = static_cast<tree_statement_list *> (ptr);
 
+  sig_atomic_t saved_octave_interrupt_state = octave_interrupt_state;
+  unwind_protect::add (restore_interrupt_state, &saved_octave_interrupt_state);
+  octave_interrupt_state = 0;
+
   // We want to run the cleanup code without error_state being set,
   // but we need to restore its value, so that any errors encountered
   // in the first part of the unwind_protect are not completely
@@ -917,9 +926,6 @@ do_unwind_protect_cleanup_code (void *ptr)
   unwind_protect_int (error_state);
   error_state = 0;
 
-  unwind_protect_int (octave_interrupt_state);
-  octave_interrupt_state = 0;
-
   // Similarly, if we have seen a return or break statement, allow all
   // the cleanup code to run before returning or handling the break.
   // We don't have to worry about continue statements because they can
@@ -980,6 +986,8 @@ do_unwind_protect_cleanup_code (void *ptr)
     unwind_protect::discard ();
   else
     unwind_protect::run ();
+
+  unwind_protect::run ();
 }
 
 void
