@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2003,
-              2005, 2006, 2007 John W. Eaton
+              2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -138,7 +138,7 @@ w32_set_octave_home (void)
     {
       size_t pos = bin_dir.rfind ("\\bin\\");
 
-      if (pos != NPOS)
+      if (pos != std::string::npos)
 	octave_env::putenv ("OCTAVE_HOME", bin_dir.substr (0, pos));
     }
 }
@@ -188,39 +188,6 @@ MSVC_init (void)
 }
 #endif
 
-#if defined (__CYGWIN__)
-
-#include <limits.h>
-#include <sys/cygwin.h>
-
-static void
-CYGWIN_init (void)
-{
-  // Make sure TMPDIR contains an absolute windows path.  Use '/'
-  // rather than '\\' so that sh expansion won't mess file names.
-
-  std::string tmpdir = octave_env::getenv ("TMPDIR");
-
-  if (tmpdir.empty ())
-    tmpdir = "/tmp";
-
-  char buf [PATH_MAX];
-
-  if (cygwin32_conv_to_full_win32_path (tmpdir.c_str (), buf) < 0)
-    panic ("CYGWIN_init: unable to convert TMPDIR (= \"%s\") to Windows directory name",
-	   tmpdir.c_str ());
-  else
-    {
-      tmpdir = buf;
-
-      for (size_t i = 0; i < tmpdir.length (); i++)
-	if (tmpdir[i] == '\\')
-	  tmpdir[i] = '/';
-    }
-
-  octave_env::putenv ("TMPDIR", tmpdir);
-}
-#endif
 
 // Return TRUE if FILE1 and FILE2 refer to the same (physical) file.
 
@@ -335,8 +302,6 @@ sysdep_init (void)
 {
 #if defined (__386BSD__) || defined (__FreeBSD__)
   BSD_init ();
-#elif defined (__CYGWIN__)
-  CYGWIN_init ();
 #elif defined (__MINGW32__)
   MINGW_init ();
 #elif defined (_MSC_VER)
@@ -427,7 +392,7 @@ raw_mode (bool on, bool wait)
 	s = save_term;
       }
 
-    tcsetattr (tty_fd, TCSAFLUSH, &s);
+    tcsetattr (tty_fd, wait ? TCSAFLUSH : TCSADRAIN, &s);
   }
 #elif defined (HAVE_TERMIO_H)
   {
@@ -660,7 +625,7 @@ DEFALIAS (setenv, putenv);
 DEFUN (kbhit, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} kbhit ()\n\
-Read a single keystroke from the keyboard. If called with one\n\
+Read a single keystroke from the keyboard.  If called with one\n\
 argument, don't wait for a keypress.  For example,\n\
 \n\
 @example\n\
@@ -905,7 +870,7 @@ tilde_expand (\"~/bin\")\n\
 
 #if defined (__EMX__) && defined (OS2)
 
-DEFCMD (extproc, , ,
+DEFUN (extproc, , ,
   "extproc: ignored by Octave")
 {
   return octave_value_list ();
