@@ -108,10 +108,9 @@
      END_OF_INPUT = 324,
      LEXICAL_ERROR = 325,
      FCN = 326,
-     VARARGIN = 327,
-     VARARGOUT = 328,
-     CLOSE_BRACE = 329,
-     UNARY = 330
+     SCRIPT = 327,
+     CLOSE_BRACE = 328,
+     UNARY = 329
    };
 #endif
 /* Tokens.  */
@@ -184,17 +183,16 @@
 #define END_OF_INPUT 324
 #define LEXICAL_ERROR 325
 #define FCN 326
-#define VARARGIN 327
-#define VARARGOUT 328
-#define CLOSE_BRACE 329
-#define UNARY 330
+#define SCRIPT 327
+#define CLOSE_BRACE 328
+#define UNARY 329
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 339 "parse.y"
+#line 364 "parse.y"
 {
   // The type of the basic tokens returned by the lexer.
   token *tok_val;
@@ -231,7 +229,7 @@ typedef union YYSTYPE
   octave_user_function *octave_user_function_type;
 }
 /* Line 1489 of yacc.c.  */
-#line 235 "y.tab.h"
+#line 233 "y.tab.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
